/*
 * AutomatonCompare.h
 *
 *  Created on: Apr 1, 2013
 *      Author: honza
 */

#ifndef AUTOMATON_COMPARE_H_
#define AUTOMATON_COMPARE_H_

#include <ostream>

#include <automaton/Automaton.h>
#include <set>
#include <list>
#include <map>
#include <utility>

class AutomatonCompare : public automaton::VisitableAutomatonBase::const_same_visitor_type {
	static bool testCompare(const automaton::DFA& a, const automaton::DFA& b);
	static void printCompare(const automaton::DFA& a, const automaton::DFA& b);

	static bool testCompare(const automaton::NFA& a, const automaton::NFA& b);
	static void printCompare(const automaton::NFA& a, const automaton::NFA& b);

	static bool testCompare(const automaton::MultiInitialStateNFA& a, const automaton::MultiInitialStateNFA& b);
	static void printCompare(const automaton::MultiInitialStateNFA& a, const automaton::MultiInitialStateNFA& b);

	static bool testCompare(const automaton::EpsilonNFA& a, const automaton::EpsilonNFA& b);
	static void printCompare(const automaton::EpsilonNFA& a, const automaton::EpsilonNFA& b);

	static bool testCompare(const automaton::ExtendedNFA& a, const automaton::ExtendedNFA& b);
	static void printCompare(const automaton::ExtendedNFA& a, const automaton::ExtendedNFA& b);

	static bool testCompare(const automaton::CompactNFA& a, const automaton::CompactNFA& b);
	static void printCompare(const automaton::CompactNFA& a, const automaton::CompactNFA& b);

	static bool testCompare(const automaton::DPDA& a, const automaton::DPDA& b);
	static void printCompare(const automaton::DPDA& a, const automaton::DPDA& b);

	static bool testCompare(const automaton::NPDA& a, const automaton::NPDA& b);
	static void printCompare(const automaton::NPDA& a, const automaton::NPDA& b);

	static bool testCompare(const automaton::InputDrivenDPDA& a, const automaton::InputDrivenDPDA& b);
	static void printCompare(const automaton::InputDrivenDPDA& a, const automaton::InputDrivenDPDA& b);

	static bool testCompare(const automaton::InputDrivenNPDA& a, const automaton::InputDrivenNPDA& b);
	static void printCompare(const automaton::InputDrivenNPDA& a, const automaton::InputDrivenNPDA& b);

	static bool testCompare(const automaton::VisiblyPushdownDPDA& a, const automaton::VisiblyPushdownDPDA& b);
	static void printCompare(const automaton::VisiblyPushdownDPDA& a, const automaton::VisiblyPushdownDPDA& b);

	static bool testCompare(const automaton::VisiblyPushdownNPDA& a, const automaton::VisiblyPushdownNPDA& b);
	static void printCompare(const automaton::VisiblyPushdownNPDA& a, const automaton::VisiblyPushdownNPDA& b);

	static bool testCompare(const automaton::RealTimeHeightDeterministicDPDA& a, const automaton::RealTimeHeightDeterministicDPDA& b);
	static void printCompare(const automaton::RealTimeHeightDeterministicDPDA& a, const automaton::RealTimeHeightDeterministicDPDA& b);

	static bool testCompare(const automaton::RealTimeHeightDeterministicNPDA& a, const automaton::RealTimeHeightDeterministicNPDA& b);
	static void printCompare(const automaton::RealTimeHeightDeterministicNPDA& a, const automaton::RealTimeHeightDeterministicNPDA& b);

	static bool testCompare(const automaton::SinglePopDPDA& a, const automaton::SinglePopDPDA& b);
	static void printCompare(const automaton::SinglePopDPDA& a, const automaton::SinglePopDPDA& b);

	static bool testCompare(const automaton::SinglePopNPDA& a, const automaton::SinglePopNPDA& b);
	static void printCompare(const automaton::SinglePopNPDA& a, const automaton::SinglePopNPDA& b);

	static bool testCompare(const automaton::OneTapeDTM& a, const automaton::OneTapeDTM& b);
	static void printCompare(const automaton::OneTapeDTM& a, const automaton::OneTapeDTM& b);

	template <class T> static void setCompare(const std::set<T> a, const std::set<T> b);
	template <class T> static void listCompare(const std::list<T> a, const std::list<T> b);
	template <class T, class R> static void mapCompare(const std::map<T, R> a, const std::map<T, R> b);
public:
	void Visit(void* data, const automaton::CompactNFA& first, const automaton::CompactNFA& second) const;
	void Visit(void* data, const automaton::DFA& first, const automaton::DFA& second) const;
	void Visit(void* data, const automaton::EpsilonNFA& first, const automaton::EpsilonNFA& second) const;
	void Visit(void* data, const automaton::ExtendedNFA& first, const automaton::ExtendedNFA& second) const;
	void Visit(void* data, const automaton::MultiInitialStateNFA& first, const automaton::MultiInitialStateNFA& second) const;
	void Visit(void* data, const automaton::NFA& first, const automaton::NFA& second) const;

	void Visit(void* data, const automaton::DPDA& first, const automaton::DPDA& second) const;
	void Visit(void* data, const automaton::NPDA& first, const automaton::NPDA& second) const;
	void Visit(void* data, const automaton::InputDrivenNPDA& first, const automaton::InputDrivenNPDA& second) const;
	void Visit(void* data, const automaton::InputDrivenDPDA& first, const automaton::InputDrivenDPDA& second) const;
	void Visit(void* data, const automaton::RealTimeHeightDeterministicDPDA& first, const automaton::RealTimeHeightDeterministicDPDA& second) const;
	void Visit(void* data, const automaton::RealTimeHeightDeterministicNPDA& first, const automaton::RealTimeHeightDeterministicNPDA& second) const;
	void Visit(void* data, const automaton::SinglePopNPDA& first, const automaton::SinglePopNPDA& second) const;
	void Visit(void* data, const automaton::SinglePopDPDA& first, const automaton::SinglePopDPDA& second) const;
	void Visit(void* data, const automaton::VisiblyPushdownDPDA& first, const automaton::VisiblyPushdownDPDA& second) const;
	void Visit(void* data, const automaton::VisiblyPushdownNPDA& first, const automaton::VisiblyPushdownNPDA& second) const;

	void Visit(void* data, const automaton::OneTapeDTM& first, const automaton::OneTapeDTM& second) const;

	static int compare(const automaton::DFA& a, const automaton::DFA& b);
	static int compare(const automaton::NFA& a, const automaton::NFA& b);
	static int compare(const automaton::MultiInitialStateNFA& a, const automaton::MultiInitialStateNFA& b);
	static int compare(const automaton::EpsilonNFA& a, const automaton::EpsilonNFA& b);
	static int compare(const automaton::ExtendedNFA& a, const automaton::ExtendedNFA& b);
	static int compare(const automaton::CompactNFA& a, const automaton::CompactNFA& b);

	static int compare(const automaton::DPDA& a, const automaton::DPDA& b);
	static int compare(const automaton::NPDA& a, const automaton::NPDA& b);
	static int compare(const automaton::InputDrivenDPDA& a, const automaton::InputDrivenDPDA& b);
	static int compare(const automaton::InputDrivenNPDA& a, const automaton::InputDrivenNPDA& b);
	static int compare(const automaton::RealTimeHeightDeterministicDPDA& a, const automaton::RealTimeHeightDeterministicDPDA& b);
	static int compare(const automaton::RealTimeHeightDeterministicNPDA& a, const automaton::RealTimeHeightDeterministicNPDA& b);
	static int compare(const automaton::SinglePopDPDA& a, const automaton::SinglePopDPDA& b);
	static int compare(const automaton::SinglePopNPDA& a, const automaton::SinglePopNPDA& b);
	static int compare(const automaton::VisiblyPushdownDPDA& a, const automaton::VisiblyPushdownDPDA& b);
	static int compare(const automaton::VisiblyPushdownNPDA& a, const automaton::VisiblyPushdownNPDA& b);

	static int compare(const automaton::OneTapeDTM& a, const automaton::OneTapeDTM& b);

	static int compare(const automaton::Automaton& a, const automaton::Automaton& b);

	static const AutomatonCompare AUTOMATON_COMPARE;
};

#endif /* AUTOMATON_COMPARE_H_ */
