/*
 * aconversion.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>

#include <sax/SaxParseInterface.h>
#include <sax/ParserException.h>
#include <factory/XmlDataFactory.hpp>

#include "ConversionHandler.h"

int main(int argc, char* argv[])
{
	try {
		TCLAP::CmdLine cmd("Converts between regular expressions, regular grammars and finite automaton.", ' ', "0.01");

		std::vector<std::string> formalisms {"fa", "re", "rg" };
		TCLAP::ValuesConstraint<std::string> allowedFormalisms( formalisms );
		TCLAP::ValueArg<std::string> target(	"t",	"target",		"Specifies target formalism",		true,	"",	&allowedFormalisms);
		cmd.add( target );

		std::vector<std::string> algorithms{"algebraic", "elimination", "brzozowski", "glushkov", "thompson", "incoming", "outgoing", "default" };
		TCLAP::ValuesConstraint<std::string> allowedAlgorithms( algorithms );
		TCLAP::ValueArg<std::string> algorithm(	"a",	"algorithm",		"Specifies algorithm to use",		false,	"default",	&allowedAlgorithms);
		cmd.add( algorithm );

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"input to convert",	false,	"-",		"file");
		cmd.add( input );

		cmd.parse(argc, argv);

		std::list<sax::Token> inputTokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(inputTokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), inputTokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(inputTokens);
		}

		ConversionHandler conv( inputTokens, target.getValue(), algorithm.getValue() );
		conv.convert( );
		return 0;
	} catch(const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
