/*
 * GasTexConverter.cpp
 *
 *  Created on: Jan 1, 2014
 *      Author: Martin Zak
 */

#include "GasTexConverter.h"

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/PDA/DPDA.h>
#include <automaton/PDA/SinglePopDPDA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/InputDrivenNPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/VisiblyPushdownNPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicNPDA.h>
#include <automaton/PDA/NPDA.h>
#include <automaton/PDA/SinglePopNPDA.h>
#include <automaton/TM/OneTapeDTM.h>

#include <factory/StringDataFactory.hpp>

#include <set>
#include <map>
#include <typeinfo>
#include "exception/AlibException.h"

void GasTexConverter::convert(const automaton::Automaton& a, std::ostream& out) {
	a.getData().Accept((void*) &out, GasTexConverter::instance);
}

void GasTexConverter::convert(const automaton::EpsilonNFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::MultiInitialStateNFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::NFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::DFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::ExtendedNFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::CompactNFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::DPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state) {
			initial = true;
		}
		if(a.getFinalStates().count(state)) {
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::SinglePopDPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state) {
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::InputDrivenDPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::InputDrivenNPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::VisiblyPushdownDPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::VisiblyPushdownNPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::RealTimeHeightDeterministicDPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::RealTimeHeightDeterministicNPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::NPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::SinglePopNPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::OneTapeDTM& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

std::string GasTexConverter::getStackSymbols(const std::vector<alphabet::Symbol>& stackSymbols) {
	if (stackSymbols.size() == 0) {
		return "$\\varepsilon$";
	}

	std::string symbols = "";
	int i=0;
	for (const auto& symbol : stackSymbols) {
		if(i++ !=0) {
			symbols +=" ";
		}
		symbols += (std::string) symbol.getData();
	}
	return symbols;
}

void GasTexConverter::printTransitionMap(const std::map<std::pair<std::string, std::string>, std::string> transitionMap, std::ostream& out) {
	for (const auto& transition : transitionMap) {
		std::string from = transition.first.first;
		std::string to = transition.first.second;
		std::string symbols = transition.second;

		if (from == to) {
			out << "\\drawloop(";
			out << from;
			out << "){";
			out << symbols;
			out << "}\n";

		} else {
			out << "\\drawedge(";
			out << from;
			out << ",";
			out << to;
			out << "){";
			out << symbols;
			out << "}\n";
		}
	}
}

void GasTexConverter::transitions(const automaton::EpsilonNFA& fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.getName());

			std::string symbol;
			if (transition.first.second.is<string::Epsilon>()) {
				symbol = "$\\varepsilon$";
			} else {
				symbol = (std::string) transition.first.second.get<alphabet::Symbol>();
			}

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::MultiInitialStateNFA& fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.getName());

			std::string symbol = (std::string) transition.first.second;

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::NFA& fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.getName());

			std::string symbol = (std::string) transition.first.second;

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::DFA& fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) transition.second.getName());

		std::string symbol = (std::string) transition.first.second;

		auto mapIterator = transitionMap.find(key);
		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(make_pair(key, symbol));
		} else {
			mapIterator->second += ", " + symbol;
		}
	}
	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::ExtendedNFA& fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.getName());

			std::string symbol = alib::StringDataFactory::toString(transition.first.second);

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::CompactNFA& fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.getName());

			std::string symbol = alib::StringDataFactory::toString(string::String(transition.first.second));

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::DPDA& pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getTransitions()) {
		std::pair<std::string, std::string> key((std::string) std::get<0>(transition.first).getName(), (std::string) transition.second.first.getName());
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		if (std::get<1>(transition.first).is<string::Epsilon>()) {
			symbol = "$\\varepsilon;$";
		} else {
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).get<alphabet::Symbol>());
		}

		symbol += "|";

		symbol += getStackSymbols(std::get<2>(transition.first));
		symbol += "\\rarrow";
		symbol += getStackSymbols(transition.second.second);

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::SinglePopDPDA& pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getTransitions()) {
		std::pair<std::string, std::string> key((std::string) std::get<0>(transition.first).getName(), (std::string) transition.second.first.getName());
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		if (std::get<1>(transition.first).is<string::Epsilon>()) {
			symbol = "$\\varepsilon;$";
		} else {
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).get<alphabet::Symbol>());
		}

		symbol += "|";

		symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
		symbol += "\\rarrow";
		symbol += getStackSymbols(transition.second.second);

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::InputDrivenDPDA& pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	const auto& symbolToPDSOperation = pda.getPushdownStoreOperations();
	for (const auto& transition : pda.getTransitions()) {
		const auto& pop = symbolToPDSOperation.find(std::get<1>(transition.first))->second.first;
		const auto& push = symbolToPDSOperation.find(std::get<1>(transition.first))->second.second;

		const auto& to = transition.second;
		std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.getName());
		auto mapIterator = transitionMap.find(key);

		std::string symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += "|";

		symbol += getStackSymbols(pop);
		symbol += "\\rarrow";
		symbol += getStackSymbols(push);

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::InputDrivenNPDA& pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	const auto& symbolToPDSOperation = pda.getPushdownStoreOperations();
	for (const auto& transition : pda.getTransitions()) {
		const auto& pop = symbolToPDSOperation.find(std::get<1>(transition.first))->second.first;
		const auto& push = symbolToPDSOperation.find(std::get<1>(transition.first))->second.second;

		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.getName());
			auto mapIterator = transitionMap.find(key);

			std::string symbol = alib::StringDataFactory::toString(transition.first.second);

			symbol += "|";

			symbol += getStackSymbols(pop);
			symbol += "\\rarrow";
			symbol += getStackSymbols(push);

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::VisiblyPushdownDPDA& pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getCallTransitions()) {
		std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) transition.second.first.getName());
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += "|";

		symbol += "$\\varepsilon;$";
		symbol += "\\rarrow";
		symbol += alib::StringDataFactory::toString(transition.second.second);

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		std::pair<std::string, std::string> key((std::string) std::get<0>(transition.first).getName(), (std::string) transition.second.getName());
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		symbol = alib::StringDataFactory::toString(std::get<1>(transition.first));

		symbol += "|";

		symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
		symbol += "\\rarrow";
		symbol += "$\\varepsilon;$";

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) transition.second.getName());
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += "|";

		symbol += "$\\varepsilon;$";
		symbol += "\\rarrow";
		symbol += "$\\varepsilon;$";

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::VisiblyPushdownNPDA& pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getCallTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.first.getName());
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			symbol = alib::StringDataFactory::toString(transition.first.second);

			symbol += "|";

			symbol += "$\\varepsilon;$";
			symbol += "\\rarrow";
			symbol += alib::StringDataFactory::toString(to.second);

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) std::get<0>(transition.first).getName(), (std::string) to.getName());
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first));

			symbol += "|";

			symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
			symbol += "\\rarrow";
			symbol += "$\\varepsilon;$";

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.getName());
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			symbol = alib::StringDataFactory::toString(transition.first.second);

			symbol += "|";

			symbol += "$\\varepsilon;$";
			symbol += "\\rarrow";
			symbol += "$\\varepsilon;$";

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::RealTimeHeightDeterministicDPDA& pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getCallTransitions()) {
		std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) transition.second.first.getName());
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		if(transition.first.second.is<string::Epsilon>())
			symbol = "$\\varepsilon;$";
		else
			symbol = alib::StringDataFactory::toString(transition.first.second.get<alphabet::Symbol>());

		symbol += "|";

		symbol += "$\\varepsilon;$";
		symbol += "\\rarrow";
		symbol += alib::StringDataFactory::toString(transition.second.second);

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		std::pair<std::string, std::string> key((std::string) std::get<0>(transition.first).getName(), (std::string) transition.second.getName());
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		if(std::get<1>(transition.first).is<string::Epsilon>())
			symbol = "$\\varepsilon;$";
		else
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).get<alphabet::Symbol>());

		symbol += "|";

		symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
		symbol += "\\rarrow";
		symbol += "$\\varepsilon;$";

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) transition.second.getName());
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		if(transition.first.second.is<string::Epsilon>())
			symbol = "$\\varepsilon;$";
		else
			symbol = alib::StringDataFactory::toString(transition.first.second.get<alphabet::Symbol>());

		symbol += "|";

		symbol += "$\\varepsilon;$";
		symbol += "\\rarrow";
		symbol += "$\\varepsilon;$";

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::RealTimeHeightDeterministicNPDA& pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getCallTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.first.getName());
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			if(transition.first.second.is<string::Epsilon>())
				symbol = "$\\varepsilon;$";
			else
				symbol = alib::StringDataFactory::toString(transition.first.second.get<alphabet::Symbol>());

			symbol += "|";

			symbol += "$\\varepsilon;$";
			symbol += "\\rarrow";
			symbol += alib::StringDataFactory::toString(to.second);

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) std::get<0>(transition.first).getName(), (std::string) to.getName());
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			if(std::get<1>(transition.first).is<string::Epsilon>())
				symbol = "$\\varepsilon;$";
			else
				symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).get<alphabet::Symbol>());

			symbol += "|";

			symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
			symbol += "\\rarrow";
			symbol += "$\\varepsilon;$";

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) to.getName());
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			if(transition.first.second.is<string::Epsilon>())
				symbol = "$\\varepsilon;$";
			else
				symbol = alib::StringDataFactory::toString(transition.first.second.get<alphabet::Symbol>());

			symbol += "|";

			symbol += "$\\varepsilon;$";
			symbol += "\\rarrow";
			symbol += "$\\varepsilon;$";

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::NPDA& pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) std::get<0>(transition.first).getName(), (std::string) to.first.getName());
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			if (std::get<1>(transition.first).is<string::Epsilon>()) {
				symbol = "$\\varepsilon;$";
			} else {
				symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).get<alphabet::Symbol>());
			}

			symbol += "|";

			symbol += getStackSymbols(std::get<2>(transition.first));
			symbol += "\\rarrow";
			symbol += getStackSymbols(to.second);

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::SinglePopNPDA& pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key((std::string) std::get<0>(transition.first).getName(), (std::string) to.first.getName());
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			if (std::get<1>(transition.first).is<string::Epsilon>()) {
				symbol = "$\\varepsilon;$";
			} else {
				symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).get<alphabet::Symbol>());
			}

			symbol += "|";

			symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
			symbol += "\\rarrow";
			symbol += getStackSymbols(to.second);

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::OneTapeDTM& tm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (auto& transition : tm.getTransitions()) {
		std::pair<std::string, std::string> key((std::string) transition.first.first.getName(), (std::string) std::get<0>(transition.second).getName());
		auto mapIterator = transitionMap.find(key);

		std::string symbol = alib::StringDataFactory::toString(transition.first.second);
		symbol += "/";
		symbol += alib::StringDataFactory::toString(std::get<1>(transition.second));
		symbol += ",";
		switch(std::get<2>(transition.second)) {
		case automaton::Shift::LEFT:
			symbol += "$\\leftarrow$";
			break;
		case automaton::Shift::RIGHT:
			symbol += "$\\rightarrow$";
			break;
		case automaton::Shift::NONE:
			symbol += "$\\times$";
			break;
		default:
			throw exception::AlibException("Unexpected shift direction");
		}

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

const GasTexConverter GasTexConverter::instance;

void GasTexConverter::Visit(void* data, const automaton::EpsilonNFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::MultiInitialStateNFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::NFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::DFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::ExtendedNFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::CompactNFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::DPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::SinglePopDPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::InputDrivenDPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::InputDrivenNPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::VisiblyPushdownDPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::VisiblyPushdownNPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::RealTimeHeightDeterministicDPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::RealTimeHeightDeterministicNPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::NPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::SinglePopNPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::OneTapeDTM& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

