/*
 * adeterminize.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>

#include "automaton/Automaton.h"
#include "factory/XmlDataFactory.hpp"
#include "exception/AlibException.h"
#include "automaton/determinize/Determinize.h"

int main(int argc, char** argv) {
	try {
		TCLAP::CmdLine cmd("Automaton determinize binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Automaton to determinize",		false,	"-",		"file");
		cmd.add( input );

		cmd.parse(argc, argv);

		std::list<sax::Token> tokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		alib::XmlDataFactory::toStdout(automaton::determinize::Determinize::determinize(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
		return 0;
	} catch (const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
