/*
 * aecho.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Martin Zak
 */

#include <tclap/CmdLine.h>
#include <string>
#include <exception/AlibException.h>
#include <factory/XmlDataFactory.hpp>
#include <sax/SaxParseInterface.h>
#include <sax/ParserException.h>
#include <object/Object.h>

int main(int argc, char** argv) {
	try {
		TCLAP::CmdLine cmd("Cat binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input to echo",		false,	"-",		"file");
		cmd.add( input );

		cmd.parse(argc, argv);

		std::list<sax::Token> tokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		alib::Object object = alib::XmlDataFactory::fromTokens<alib::Object>(tokens);
		alib::XmlDataFactory::toStdout(object);

		return 0;
	} catch(const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
