/*
 * aepsilon.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/AlibException.h>

#include "automaton/simplify/EpsilonRemoverIncoming.h"
#include "automaton/simplify/EpsilonRemoverOutgoing.h"
#include "automaton/simplify/efficient/EpsilonRemoverIncoming.h"
#include "automaton/simplify/efficient/EpsilonRemoverOutgoing.h"
#include "grammar/simplify/EpsilonRemover.h"

int main(int argc, char** argv) {

	try {
		TCLAP::CmdLine cmd("Automaton rename binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Automaton/Grammar where to remove epsilon transitions",		false,	"-",		"file");
		cmd.add( input );

		std::vector<std::string> algorithms{"incoming", "outgoing", "default" };
		TCLAP::ValuesConstraint<std::string> allowedAlgorithms( algorithms );
		TCLAP::ValueArg<std::string> algorithm(	"a",	"algorithm",		"Specifies algorithm to use",		false,	"default",	&allowedAlgorithms);
		cmd.add( algorithm );

		TCLAP::SwitchArg efficient(		"e",	"efficient",		"Use efficient implementation",		false);
		cmd.add( efficient );

		cmd.parse(argc, argv);

		std::list<sax::Token> tokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		if(efficient.getValue()) {
			if(algorithm.getValue() == "outgoing") {
				alib::XmlDataFactory::toStdout(automaton::simplify::efficient::EpsilonRemoverOutgoing::remove(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
			} else if(algorithm.getValue() == "incoming") {
				alib::XmlDataFactory::toStdout(automaton::simplify::efficient::EpsilonRemoverIncoming::remove(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
			} else if(algorithm.getValue() == "default") {
				if(alib::XmlDataFactory::first<automaton::Automaton>(tokens))
					alib::XmlDataFactory::toStdout(automaton::simplify::efficient::EpsilonRemoverIncoming::remove(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
				else if(alib::XmlDataFactory::first<grammar::Grammar>(tokens))
					throw exception::AlibException("NYI");
				else
					throw exception::AlibException("Invalid formalism");
			} else {
				throw exception::AlibException("Invalid algorithm");
			}
		} else {
			if(algorithm.getValue() == "outgoing") {
				alib::XmlDataFactory::toStdout(automaton::simplify::EpsilonRemoverOutgoing::remove(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
			} else if(algorithm.getValue() == "incoming") {
				alib::XmlDataFactory::toStdout(automaton::simplify::EpsilonRemoverIncoming::remove(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
			} else if(algorithm.getValue() == "default") {
				if(alib::XmlDataFactory::first<automaton::Automaton>(tokens))
					alib::XmlDataFactory::toStdout(automaton::simplify::EpsilonRemoverIncoming::remove(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
				else if(alib::XmlDataFactory::first<grammar::Grammar>(tokens))
					alib::XmlDataFactory::toStdout(grammar::simplify::EpsilonRemover::remove(alib::XmlDataFactory::fromTokens<grammar::Grammar>(tokens)));
				else
					throw exception::AlibException("Invalid formalism");
			} else {
				throw exception::AlibException("Invalid algorithm");
			}
		}
		return 0;

	} catch (const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
