/*
 * agenerate.cpp
 *
 *  Created on: 16. 2. 2015
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <vector>

#include <factory/XmlDataFactory.hpp>
#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <grammar/Grammar.h>
#include <grammar/generate/GenerateUpToLength.h>

int main(int argc, char* argv[]) {
	try {
		TCLAP::CmdLine cmd("String generate from grammar", ' ', "0.01");

		TCLAP::ValueArg<unsigned> upto(	"",	"upto",		"Generate all strings up to length",		false,	5, "unsigned");
		cmd.add( upto );

		TCLAP::ValueArg<std::string> grammar(	"g",	"grammar",	"Grammar to use",	false,	"-",		"file");
		cmd.add( grammar );

		cmd.parse(argc, argv);

		std::list<sax::Token> grammarTokens;
		if(grammar.isSet()) {
			if(grammar.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(grammarTokens);
			} else {
				sax::SaxParseInterface::parseFile(grammar.getValue(), grammarTokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(grammarTokens);
		}

		if( upto.isSet() ) {
			grammar::Grammar grammar = alib::XmlDataFactory::fromTokens<grammar::Grammar>(grammarTokens);
			std::set<string::LinearString> res = grammar::generate::GenerateUpToLength::generate(grammar, upto.getValue());
			alib::XmlDataFactory::toStdout( res );
			return 0;
		} else {
			throw exception::AlibException( "Invalid run type" );
			return 1;
		}
	} catch ( const exception::AlibException & exception ) {
		alib::XmlDataFactory::toStdout( exception );
		return 1;
	} catch (const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch (...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
