/*
 * alangop.cpp
 *
 *  Created on: 20. 11. 2014
 *	  Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>
#include <vector>

#include <factory/XmlDataFactory.hpp>
#include <exception/AlibException.h>
#include <automaton/Automaton.h>

#include <automaton/transform/AutomataConcatenationEpsilonTransition.h>
#include <automaton/transform/AutomataConcatenation.h>
#include <automaton/transform/AutomataIntersectionCartesianProduct.h>
#include <automaton/transform/AutomataUnionCartesianProduct.h>
#include <automaton/transform/AutomataUnionEpsilonTransition.h>
#include <automaton/transform/AutomatonIterationEpsilonTransition.h>
#include <automaton/transform/AutomatonIteration.h>


int main(int argc, char* argv[]) {
	try {
		TCLAP::CmdLine cmd("Stringology algorithm access binary", ' ', "0.01");

		std::vector<std::string> allowed;
		allowed.push_back("unionEpsilon");
		allowed.push_back("unionCartesian");
		allowed.push_back("concatenationEpsilon");
		allowed.push_back("concatenation");
		allowed.push_back("intersectionCartesian");
		allowed.push_back("iterationEpsilon");
		allowed.push_back("iteration");
		TCLAP::ValuesConstraint<std::string> allowedVals( allowed );

		TCLAP::ValueArg<std::string> algorithm(	"a",	"algorithm",	"Execute algorithm", true,	"",	&allowedVals);
		cmd.add(algorithm);

		TCLAP::ValueArg<std::string> a1(	"i",	"input1",	"First automaton",	false,	"-",	"file");
		TCLAP::ValueArg<std::string> a2(	"j",	"input2",	"Second automaton",	false,	"-",	"file");
        cmd.add( a1 );
        cmd.add( a2 );

		cmd.parse(argc,argv);

		std::list<sax::Token> a1Tokens, a2Tokens;
		if(a1.isSet()) {
			if(a1.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(a1Tokens);
			} else {
				sax::SaxParseInterface::parseFile(a1.getValue(), a1Tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(a1Tokens);
		}

		if(a2.isSet()) {
			if(a2.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(a2Tokens);
			} else {
				sax::SaxParseInterface::parseFile(a2.getValue(), a2Tokens);
			}
		} else if(algorithm.getValue() != "iteration" && algorithm.getValue() != "iterationEpsilon") {
			sax::SaxParseInterface::parseStdin(a2Tokens);
		}

		automaton::Automaton automaton1 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(a1Tokens);

		if( algorithm.getValue() == "unionEpsilon") {
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(a2Tokens);
			alib::XmlDataFactory::toStdout(automaton::transform::AutomataUnionEpsilonTransition::unification(automaton1, automaton2));
			return 0;
		} else if( algorithm.getValue() == "unionCartesian") {
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(a2Tokens);
			alib::XmlDataFactory::toStdout(automaton::transform::AutomataUnionCartesianProduct::unification(automaton1, automaton2));
			return 0;
		} else if( algorithm.getValue() == "concatenationEpsilon") {
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(a2Tokens);
			alib::XmlDataFactory::toStdout(automaton::transform::AutomataConcatenationEpsilonTransition::concatenation(automaton1, automaton2));
			return 0;
		} else if( algorithm.getValue() == "concatenation") {
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(a2Tokens);
			alib::XmlDataFactory::toStdout(automaton::transform::AutomataConcatenation::concatenation(automaton1, automaton2));
			return 0;
		} else if( algorithm.getValue() == "intersectionCartesian") {
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(a2Tokens);
			alib::XmlDataFactory::toStdout(automaton::transform::AutomataIntersectionCartesianProduct::intersection(automaton1, automaton2));
			return 0;
		} else if( algorithm.getValue() == "iteration") {
			alib::XmlDataFactory::toStdout(automaton::transform::AutomatonIteration::iteration(automaton1));
			return 0;
		} else if( algorithm.getValue() == "iterationEpsilon") {
			alib::XmlDataFactory::toStdout(automaton::transform::AutomatonIterationEpsilonTransition::iteration(automaton1));
			return 0;
		} else {
			throw exception::AlibException( "Invalid algorithm" );
			return 1;
		}
	} catch( const exception::AlibException & exception ) {
		alib::XmlDataFactory::toStdout( exception );
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
