/*
 * UnreachableStates.h
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef UNREACHABLE_STATES_H_
#define UNREACHABLE_STATES_H_

#include <algorithm>
#include <deque>
#include <set>

#include <automaton/common/State.h>
#include <automaton/Automaton.h>

namespace automaton {

namespace properties {

class UnreachableStates : public VisitableAutomatonBase::const_visitor_type {
public:
	static std::set<automaton::State> unreachableStates( const automaton::Automaton & automaton );

	/**
	 * Removes dead states from FSM. Melichar 2.29
	 */
	template<class T>
	static std::set<automaton::State> unreachableStates( const T & fsm );

private:
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const MultiInitialStateNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;

	static const UnreachableStates UNREACHABLE_STATES;
};

} /* namespace properties */

} /* namespace automaton */

#endif /* UNREACHABLE_STATES_H_ */
