/*
 * ToCNF.h
 *
 *  Created on: 24. 11. 2014
 *	  Author: Jan Travnicek
 */

#ifndef TO_CNF_H_
#define TO_CNF_H_

#include <map>
#include <algorithm>

#include <grammar/Grammar.h>

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>
#include <exception/AlibException.h>

namespace grammar {

namespace simplify {

class ToCNF : public grammar::VisitableGrammarBase::const_visitor_type {
public:
	static grammar::Grammar convert( const grammar::Grammar & grammar );

	static grammar::CNF convert( const grammar::CFG & grammar );
	static grammar::CNF convert( const grammar::EpsilonFreeCFG & grammar );
	static grammar::CNF convert( const grammar::GNF & grammar );
	static grammar::CNF convert( const grammar::CNF & grammar );
	static grammar::CNF convert( const grammar::LG & grammar );
	static grammar::CNF convert( const grammar::LeftLG & grammar );
	static grammar::CNF convert( const grammar::LeftRG & grammar );
	static grammar::CNF convert( const grammar::RightLG & grammar );
	static grammar::CNF convert( const grammar::RightRG & grammar );

private:
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	static const ToCNF TO_CNF;
};

} /* namespace simplify */

} /* namespace grammar */

#endif /* TO_CNF_H_ */
