#ifndef GRAPH_BELLMAN_FORD_H_
#define GRAPH_BELLMAN_FORD_H_

#include <unordered_map>

#include <graph/Graph.h>
#include <graph/directed/DirectedGraph.h>
#include <graph/undirected/UndirectedGraph.h>

namespace graph
{

namespace shortestpath
{

// Bellman-Ford only works on graphs without negative-weight cycle
// However it will detect the negative-weight cycle and throw exception
//
// note: negative-weight undirected edge = negative-weight cycle

class BellmanFord : public graph::VisitableGraphBase::const_visitor_type
{
public:
	typedef std::unordered_map<Node, int> Result;

	static Result bellmanford(const Graph &graph, const Node &start);

	static Result bellmanford(const DirectedGraph &graph, const Node &start);
	static Result bellmanford(const UndirectedGraph &graph, const Node &start);

private:
	void Visit(void *data, const DirectedGraph &graph) const;
	void Visit(void *data, const UndirectedGraph &graph) const;

	static const BellmanFord BELLMAN_FORD;
};

} // namespace shortestpath

} // namespace graph

#endif // GRAPH_BELLMAN_FORD_H_
