#ifndef GRAPH_DIJKSTRA_H_
#define GRAPH_DIJKSTRA_H_

#include <unordered_map>

#include <graph/Graph.h>
#include <graph/directed/DirectedGraph.h>
#include <graph/undirected/UndirectedGraph.h>

namespace graph
{

namespace shortestpath
{

// Dijkstra only works on graphs without negative-weight edges (>= 0)

class Dijkstra : public graph::VisitableGraphBase::const_visitor_type
{
public:
	typedef std::unordered_map<Node, int> Result;

	static Result dijkstra(const Graph &graph, const Node &start);

	static Result dijkstra(const DirectedGraph &graph, const Node &start);
	static Result dijkstra(const UndirectedGraph &graph, const Node &start);

private:
	void Visit(void *data, const DirectedGraph &graph) const;
	void Visit(void *data, const UndirectedGraph &graph) const;

	static const Dijkstra DIJKSTRA;
};

} // namespace shortestpath

} // namespace graph

#endif // GRAPH_DIJKSTRA_H_
