#ifndef GRAPH_DFS_H_
#define GRAPH_DFS_H_

#include <graph/Graph.h>
#include <graph/directed/DirectedGraph.h>
#include <graph/undirected/UndirectedGraph.h>

namespace graph
{

namespace traverse
{

// func is called for each visited node, traversal stops when returning false
//
// bool Function(const Node &node);
// bool FunctionExt(const Node &node, const Node &predecessor, int openTime, int closeTime);

class Dfs : public graph::VisitableGraphBase::const_visitor_type
{
public:
	typedef std::function<bool(const Node&)> Function;
	typedef std::function<bool(const Node&, const Node&, int, int)> FunctionExt;

	static void dfs(const Graph &graph, const Node &start, Function func);
	static void dfs(const Graph &graph, const Node &start, FunctionExt func);

	static void dfs(const DirectedGraph &graph, const Node &start, Function func);
	static void dfs(const DirectedGraph &graph, const Node &start, FunctionExt func);
	static void dfs(const UndirectedGraph &graph, const Node &start, Function func);
	static void dfs(const UndirectedGraph &graph, const Node &start, FunctionExt func);

private:
	void Visit(void *data, const DirectedGraph &graph) const;
	void Visit(void *data, const UndirectedGraph &graph) const;

	static const Dfs DFS;
};

} // namespace traverse

} // namespace graph

#endif // GRAPH_DFS_H_
