/*
 * NormalizeAlphabet.cpp
 *
 *  Created on: Dec 9, 2013
 *	Author: Jan Travnicek
 */

#include "NormalizeAlphabet.h"

#include "std/map.hpp"
#include <deque>
#include <vector>
#include <set>
#include <algorithm>
#include <sstream>
#include <iostream>
#include "exception/AlibException.h"

#include "alphabet/Symbol.h"

namespace string {

namespace simplify {

string::String NormalizeAlphabet::normalize(const string::String& string) {
	string::String* out = NULL;
	string.getData().Accept((void*) &out, NormalizeAlphabet::NORMALIZE);
	string::String res = std::move(*out);
	delete out;
	return res;
}

string::LinearString NormalizeAlphabet::normalize(const string::LinearString& string) {
	int counter = 0;
	std::map<alphabet::Symbol, char > normalizationData;

	for(const alphabet::Symbol& symbol : string.getContent()) {
		if(normalizationData.find(symbol) == normalizationData.end()) {
			normalizationData.insert(std::make_pair(symbol, counter++));
		}
	}

	std::set<alphabet::Symbol> alphabet;
	for(const auto& symbol : normalizationData) {
		alphabet.insert(alphabet::symbolFrom((char) (symbol.second + 'a')));
	}

	std::vector<alphabet::Symbol> data;
	for(const alphabet::Symbol& symbol : string.getContent()) {
		data.push_back(alphabet::symbolFrom((char) (normalizationData.find(symbol)->second + 'a')));
	}

	string::LinearString result(alphabet, data);
	return result;
}

void NormalizeAlphabet::Visit(void*, const string::Epsilon&) const {
	throw exception::AlibException("Unsupported automaton type Epsilon");
}

void NormalizeAlphabet::Visit(void* data, const string::LinearString& str) const {
	string::String* & out = *((string::String**) data);
	out = new string::String(this->normalize(str));
}

void NormalizeAlphabet::Visit(void*, const string::CyclicString&) const {
	throw exception::AlibException("Unsupported automaton type CyclicString");
}

const NormalizeAlphabet NormalizeAlphabet::NORMALIZE;

} /* namespace simplify */

} /* namespace string */
