/*
 * ExactMatch.cpp
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#include "ExactMatch.h"
#include <exception/AlibException.h>
#include <string/LinearString.h>

#include <deque>

namespace stringology {

namespace exact {

std::set<unsigned> ExactMatch::match(const string::String& subject, const string::String& pattern) {
	std::set<unsigned> data;
	Accept((void*) &data, subject.getData(), pattern.getData(), ExactMatch::EXACT_MATCH);
	return data;
}

std::set<unsigned> ExactMatch::match(const string::LinearString& subject, const string::LinearString& pattern) {
	std::set<unsigned> occ;
	for(unsigned i = 0; i <= subject.getContent().size() - pattern.getContent().size(); i++) {
		unsigned j = 0;
		for(; j < pattern.getContent().size(); j++) {
			if(pattern.getContent()[j] != subject.getContent()[i+j]) break;
		}

		if( j == pattern.getContent().size() )
			occ.insert(i);
	}
	return occ;
}

void ExactMatch::Visit(void*, const string::Epsilon&, const string::Epsilon&) const {
	throw exception::AlibException("Unsupported string type Epsilon");
}

void ExactMatch::Visit(void* data, const string::LinearString& subject, const string::LinearString& pattern) const {
	std::set<unsigned> & res = *((std::set<unsigned>*) data);
	res = this->match(subject, pattern);
}

void ExactMatch::Visit(void*, const string::CyclicString&, const string::CyclicString&) const {
	throw exception::AlibException("Unsupported string type CyclicString");
}

const ExactMatch ExactMatch::EXACT_MATCH;

} /* namespace exact */

} /* namespace stringology */
