/*
 * StringApi.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#include "StringApi.hpp"

#include "exception/AlibException.h"

namespace alib {

const label::LabelFromStringParser FromStringParsers::labelParser;
const alphabet::SymbolFromStringParser FromStringParsers::symbolParser;
const regexp::RegExpFromStringParser FromStringParsers::regexpParser;
const string::StringFromStringParser FromStringParsers::stringParser;
const automaton::AutomatonFromStringParser FromStringParsers::automatonParser;
const grammar::GrammarFromStringParser FromStringParsers::grammarParser;
const graph::GraphFromStringParser FromStringParsers::graphParser;
const primitive::PrimitiveFromStringParser FromStringParsers::primitiveParser;

const label::LabelToStringComposer ToStringComposers::labelComposer;
const alphabet::SymbolToStringComposer ToStringComposers::symbolComposer;
const regexp::RegExpToStringComposer ToStringComposers::regexpComposer;
const string::StringToStringComposer ToStringComposers::stringComposer;
const automaton::AutomatonToStringComposer ToStringComposers::automatonComposer;
const grammar::GrammarToStringComposer ToStringComposers::grammarComposer;
const graph::GraphToStringComposer ToStringComposers::graphComposer;
const primitive::PrimitiveToStringComposer ToStringComposers::primitiveComposer;

alphabet::Symbol stringApi<alphabet::Symbol>::parse(std::istream& input) {
	return FromStringParsers::symbolParser.parseSymbol(input);
}

void stringApi<alphabet::Symbol>::compose(std::ostream& output, const alphabet::Symbol& data) {
	return ToStringComposers::symbolComposer.compose(output, data);
}


automaton::Automaton stringApi<automaton::Automaton>::parse(std::istream& input) {
	return FromStringParsers::automatonParser.parseAutomaton(input);
}

void stringApi<automaton::Automaton>::compose(std::ostream& output, const automaton::Automaton& data) {
	return ToStringComposers::automatonComposer.compose(output, data);
}

grammar::Grammar stringApi<grammar::Grammar>::parse(std::istream& input) {
	return FromStringParsers::grammarParser.parseGrammar(input);
}

void stringApi<grammar::Grammar>::compose(std::ostream& output, const grammar::Grammar& data) {
	return ToStringComposers::grammarComposer.compose(output, data);
}

graph::Graph stringApi<graph::Graph>::parse(std::istream& input) {
	return FromStringParsers::graphParser.parseGraph(input);
}

void stringApi<graph::Graph>::compose(std::ostream& output, const graph::Graph& data) {
	return ToStringComposers::graphComposer.compose(output, data);
}

label::Label stringApi<label::Label>::parse(std::istream& input) {
	return FromStringParsers::labelParser.parseLabel(input);
}

void stringApi<label::Label>::compose(std::ostream& output, const label::Label& data) {
	return ToStringComposers::labelComposer.compose(output, data);
}

regexp::RegExp stringApi<regexp::RegExp>::parse(std::istream& input) {
	return FromStringParsers::regexpParser.parseRegExp(input);
}

void stringApi<regexp::RegExp>::compose(std::ostream& output, const regexp::RegExp& data) {
	return ToStringComposers::regexpComposer.compose(output, data);
}

string::String stringApi<string::String>::parse(std::istream& input) {
	return FromStringParsers::stringParser.parseString(input);
}

void stringApi<string::String>::compose(std::ostream& output, const string::String& data) {
	return ToStringComposers::stringComposer.compose(output, data);
}


primitive::Primitive stringApi<primitive::Primitive>::parse(std::istream& input) {
	return FromStringParsers::primitiveParser.parsePrimitive(input);
}

void stringApi<primitive::Primitive>::compose(std::ostream& output, const primitive::Primitive& data) {
	return ToStringComposers::primitiveComposer.compose(output, data);
}

} /* namespace alib */
