/*
 * StringApi.hpp
 *
 * Created on: Dec 27, 2014
 * Author: Jan Travnicek
 */

#ifndef STRING_API_HPP_
#define STRING_API_HPP_

#include "label/LabelFromStringParser.h"
#include "alphabet/SymbolFromStringParser.h"
#include "regexp/RegExpFromStringParser.h"
#include "string/StringFromStringParser.h"
#include "automaton/AutomatonFromStringParser.h"
#include "grammar/GrammarFromStringParser.h"
#include "graph/GraphFromStringParser.h"
#include "primitive/PrimitiveFromStringParser.h"

#include "label/LabelToStringComposer.h"
#include "alphabet/SymbolToStringComposer.h"
#include "regexp/RegExpToStringComposer.h"
#include "string/StringToStringComposer.h"
#include "automaton/AutomatonToStringComposer.h"
#include "grammar/GrammarToStringComposer.h"
#include "graph/GraphToStringComposer.h"
#include "primitive/PrimitiveToStringComposer.h"

#include "object/ObjectBase.h"

namespace alib {

template<typename T>
struct stringApi {
	static T parse(std::istream& input);
	static void compose(std::ostream& output, const T& data);
};

template<>
struct stringApi<alphabet::Symbol> {
	static alphabet::Symbol parse(std::istream& input);
	static void compose(std::ostream& output, const alphabet::Symbol& data);
};


template<>
struct stringApi<automaton::Automaton> {
	static automaton::Automaton parse(std::istream& input);
	static void compose(std::ostream& output, const automaton::Automaton& data);
};

template<>
struct stringApi<grammar::Grammar> {
	static grammar::Grammar parse(std::istream& input);
	static void compose(std::ostream& output, const grammar::Grammar& data);
};

template<>
struct stringApi<graph::Graph> {
	static graph::Graph parse(std::istream& input);
	static void compose(std::ostream& output, const graph::Graph& data);
};

template<>
struct stringApi<label::Label> {
	static label::Label parse(std::istream& input);
	static void compose(std::ostream& output, const label::Label& data);
};

template<>
struct stringApi<regexp::RegExp> {
	static regexp::RegExp parse(std::istream& input);
	static void compose(std::ostream& output, const regexp::RegExp& data);
};

template<>
struct stringApi<string::String> {
	static string::String parse(std::istream& input);
	static void compose(std::ostream& output, const string::String& data);
};

template<>
struct stringApi<primitive::Primitive> {
	static primitive::Primitive parse(std::istream& input);
	static void compose(std::ostream& output, const primitive::Primitive& data);
};

class FromStringParsers {
public:
	static const label::LabelFromStringParser labelParser;
	static const alphabet::SymbolFromStringParser symbolParser;
	static const regexp::RegExpFromStringParser regexpParser;
	static const string::StringFromStringParser stringParser;
	static const automaton::AutomatonFromStringParser automatonParser;
	static const grammar::GrammarFromStringParser grammarParser;
	static const graph::GraphFromStringParser graphParser;
	static const primitive::PrimitiveFromStringParser primitiveParser;

};

class ToStringComposers {
public:
	static const label::LabelToStringComposer labelComposer;
	static const alphabet::SymbolToStringComposer symbolComposer;
	static const regexp::RegExpToStringComposer regexpComposer;
	static const string::StringToStringComposer stringComposer;
	static const automaton::AutomatonToStringComposer automatonComposer;
	static const grammar::GrammarToStringComposer grammarComposer;
	static const graph::GraphToStringComposer graphComposer;
	static const primitive::PrimitiveToStringComposer primitiveComposer;
};

} /* namespace alib */

#endif /* STRING_API_HPP_ */
