/*
 * XmlApi.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef XML_API_HPP_
#define XML_API_HPP_

#include "label/LabelFromXMLParser.h"
#include "alphabet/SymbolFromXMLParser.h"
#include "regexp/RegExpFromXMLParser.h"
#include "string/StringFromXMLParser.h"
#include "automaton/AutomatonFromXMLParser.h"
#include "grammar/GrammarFromXMLParser.h"
#include "graph/GraphFromXMLParser.h"
#include "object/ObjectFromXMLParser.h"
#include "exception/ExceptionFromXMLParser.h"
#include "primitive/PrimitiveFromXMLParser.h"

#include "label/LabelToXMLComposer.h"
#include "alphabet/SymbolToXMLComposer.h"
#include "regexp/RegExpToXMLComposer.h"
#include "string/StringToXMLComposer.h"
#include "automaton/AutomatonToXMLComposer.h"
#include "grammar/GrammarToXMLComposer.h"
#include "graph/GraphToXMLComposer.h"
#include "object/ObjectToXMLComposer.h"
#include "exception/ExceptionToXMLComposer.h"
#include "primitive/PrimitiveToXMLComposer.h"

#include "object/ObjectBase.h"

namespace alib {

class Names {
public:
	const static std::string ALIB_VOID;
	const static std::string CONTAINER_OBJECTS_SET;
	const static std::string CONTAINER_OBJECTS_VECTOR;
	const static std::string CONTAINER_OBJECTS_PAIR;
	const static std::string CONTAINER_OBJECTS_MAP;
	const static std::string EXCEPTION_ALIB_EXCEPTION;
	const static std::string ALPHABET_LABELED_SYMBOL;
	const static std::string ALPHABET_BLANK_SYMBOL;
	const static std::string ALPHABET_BOTTOM_OF_THE_STACK_SYMBOL;
	const static std::string ALPHABET_END_SYMBOL;
	const static std::string ALPHABET_RANKED_SYMBOL;
	const static std::string ALPHABET_BAR_SYMBOL;
	const static std::string ALPHABET_RANKED_BAR_SYMBOL;
	const static std::string ALPHABET_SUBTREE_WILDCARD_SYMBOL;
	const static std::string ALPHABET_SYMBOL_PAIR_SYMBOL;
	const static std::string ALPHABET_SYMBOL_SET_SYMBOL;
	const static std::string ALPHABET_UNIQUE_SYMBOL;
	const static std::string ALPHABET_START_SYMBOL;
	const static std::string AUTOMATON_EPSILON_NFA;
	const static std::string AUTOMATON_MULTI_INITIAL_STATE_NFA;
	const static std::string AUTOMATON_NFA;
	const static std::string AUTOMATON_DFA;
	const static std::string AUTOMATON_EXTENDED_NFA;
	const static std::string AUTOMATON_COMPACT_NFA;
	const static std::string AUTOMATON_DPDA;
	const static std::string AUTOMATON_SINGLE_POP_DPDA;
	const static std::string AUTOMATON_INPUT_DRIVEN_DPDA;
	const static std::string AUTOMATON_INPUT_DRIVEN_NPDA;
	const static std::string AUTOMATON_VISIBLY_PUSHDOWN_DPDA;
	const static std::string AUTOMATON_VISIBLY_PUSHDOWN_NPDA;
	const static std::string AUTOMATON_REAL_TIME_HEIGHT_DETERMINISTIC_DPDA;
	const static std::string AUTOMATON_REAL_TIME_HEIGHT_DETERMINISTIC_NPDA;
	const static std::string AUTOMATON_NPDA;
	const static std::string AUTOMATON_SINGLE_POP_NPDA;
	const static std::string AUTOMATON_ONE_TAPE_DTM;
	const static std::string AUTOMATON_STATE;
	const static std::string GRAMMAR_LEFT_LG;
	const static std::string GRAMMAR_LEFT_RG;
	const static std::string GRAMMAR_RIGHT_LG;
	const static std::string GRAMMAR_RIGHT_RG;
	const static std::string GRAMMAR_LG;
	const static std::string GRAMMAR_CFG;
	const static std::string GRAMMAR_EPSILON_FREE_CFG;
	const static std::string GRAMMAR_CNF;
	const static std::string GRAMMAR_GNF;
	const static std::string GRAMMAR_CSG;
	const static std::string GRAMMAR_NON_CONTRACTING_GRAMMAR;
	const static std::string GRAMMAR_CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR;
	const static std::string GRAMMAR_UNRESTRICTED_GRAMMAR;
	const static std::string GRAPH_DIRECTED_GRAPH;
	const static std::string GRAPH_UNDIRECTED_GRAPH;
	const static std::string LABEL_PRIMITIVE_LABEL;
	const static std::string LABEL_HEXAVIGESIMAL_LABEL;
	const static std::string LABEL_OBJECT_LABEL;
	const static std::string LABEL_LABEL_SET_LABEL;
	const static std::string LABEL_LABEL_PAIR_LABEL;
	const static std::string LABEL_UNIQUE_LABEL;
	const static std::string REGEXP_UNBOUNDED_REGEXP;
	const static std::string REGEXP_FORMAL_REGEXP;
	const static std::string STRING_LINEAR_STRING;
	const static std::string STRING_CYCLIC_STRING;
	const static std::string STRING_EPSILON;
	const static std::string PRIMITIVE_STRING;
	const static std::string PRIMITIVE_INTEGER;
	const static std::string PRIMITIVE_CHARACTER;
	const static std::string PRIMITIVE_UNSIGNED;
	const static std::string PRIMITIVE_BOOL;

};

template<typename T>
struct xmlApi {
	static T parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const T& data);
};

template<>
struct xmlApi<Void> {
	static Void parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const Void& data);
};

template<typename T>
struct xmlApi<std::set<T>> {
	static std::set<T> parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const std::set<T>& data);
};

template<>
struct xmlApi<container::ObjectsSet> {
	static container::ObjectsSet parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const container::ObjectsSet& data);
};

template<typename T>
struct xmlApi<std::vector<T>> {
	static std::vector<T> parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const std::vector<T>& data);
};

template<>
struct xmlApi<container::ObjectsVector> {
	static container::ObjectsVector parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const container::ObjectsVector& data);
};

template<typename T, typename R>
struct xmlApi<std::pair<T, R>> {
	static std::pair<T, R> parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const std::pair<T, R>& data);
};

template<>
struct xmlApi<container::ObjectsPair> {
	static container::ObjectsPair parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const container::ObjectsPair& data);
};

template<typename T, typename R>
struct xmlApi<std::map<T, R>> {
	static std::map<T, R> parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const std::map<T, R>& data);
};

template<>
struct xmlApi<container::ObjectsMap> {
	static container::ObjectsMap parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const container::ObjectsMap& data);
};

/* TODO for gcc-4.9 uncomment
template<typename ... Ts>
struct xmlApi<std::variant<Ts...>> {
	static std::variant<Ts...> parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const std::variant<Ts...>& data);
};

TODO tuple

*/


template<>
struct xmlApi<Object> {
	static Object parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const Object& data);
};


template<>
struct xmlApi<exception::AlibException> {
	static exception::AlibException parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const exception::AlibException& data);
};


template<>
struct xmlApi<alphabet::Symbol> {
	static alphabet::Symbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::Symbol& data);
};

template<>
struct xmlApi<alphabet::LabeledSymbol> {
	static alphabet::LabeledSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::LabeledSymbol& data);
};

template<>
struct xmlApi<alphabet::BlankSymbol> {
	static alphabet::BlankSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::BlankSymbol& data);
};

template<>
struct xmlApi<alphabet::BottomOfTheStackSymbol> {
	static alphabet::BottomOfTheStackSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::BottomOfTheStackSymbol& data);
};

template<>
struct xmlApi<alphabet::EndSymbol> {
	static alphabet::EndSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::EndSymbol& data);
};

template<>
struct xmlApi<alphabet::StartSymbol> {
	static alphabet::StartSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::StartSymbol& data);
};

template<>
struct xmlApi<alphabet::RankedSymbol> {
	static alphabet::RankedSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::RankedSymbol& data);
};

template<>
struct xmlApi<alphabet::BarSymbol> {
	static alphabet::BarSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::BarSymbol& data);
};

template<>
struct xmlApi<alphabet::RankedBarSymbol> {
	static alphabet::RankedBarSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::RankedBarSymbol& data);
};

template<>
struct xmlApi<alphabet::SubtreeWildcardSymbol> {
	static alphabet::SubtreeWildcardSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::SubtreeWildcardSymbol& data);
};

template<>
struct xmlApi<alphabet::SymbolSetSymbol> {
	static alphabet::SymbolSetSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::SymbolSetSymbol& data);
};

template<>
struct xmlApi<alphabet::SymbolPairSymbol> {
	static alphabet::SymbolPairSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::SymbolPairSymbol& data);
};

template<>
struct xmlApi<alphabet::UniqueSymbol> {
	static alphabet::UniqueSymbol parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const alphabet::UniqueSymbol& data);
};


template<>
struct xmlApi<automaton::Automaton> {
	static automaton::Automaton parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::Automaton& data);
};

template<>
struct xmlApi<automaton::EpsilonNFA> {
	static automaton::EpsilonNFA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::EpsilonNFA& data);
};

template<>
struct xmlApi<automaton::MultiInitialStateNFA> {
	static automaton::MultiInitialStateNFA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::MultiInitialStateNFA& data);
};

template<>
struct xmlApi<automaton::NFA> {
	static automaton::NFA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::NFA& data);
};

template<>
struct xmlApi<automaton::DFA> {
	static automaton::DFA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::DFA& data);
};

template<>
struct xmlApi<automaton::ExtendedNFA> {
	static automaton::ExtendedNFA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::ExtendedNFA& data);
};

template<>
struct xmlApi<automaton::CompactNFA> {
	static automaton::CompactNFA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::CompactNFA& data);
};

template<>
struct xmlApi<automaton::DPDA> {
	static automaton::DPDA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::DPDA& data);
};

template<>
struct xmlApi<automaton::SinglePopDPDA> {
	static automaton::SinglePopDPDA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::SinglePopDPDA& data);
};

template<>
struct xmlApi<automaton::InputDrivenDPDA> {
	static automaton::InputDrivenDPDA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::InputDrivenDPDA& data);
};

template<>
struct xmlApi<automaton::InputDrivenNPDA> {
	static automaton::InputDrivenNPDA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::InputDrivenNPDA& data);
};

template<>
struct xmlApi<automaton::VisiblyPushdownDPDA> {
	static automaton::VisiblyPushdownDPDA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::VisiblyPushdownDPDA& data);
};

template<>
struct xmlApi<automaton::VisiblyPushdownNPDA> {
	static automaton::VisiblyPushdownNPDA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::VisiblyPushdownNPDA& data);
};

template<>
struct xmlApi<automaton::RealTimeHeightDeterministicDPDA> {
	static automaton::RealTimeHeightDeterministicDPDA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::RealTimeHeightDeterministicDPDA& data);
};

template<>
struct xmlApi<automaton::RealTimeHeightDeterministicNPDA> {
	static automaton::RealTimeHeightDeterministicNPDA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::RealTimeHeightDeterministicNPDA& data);
};

template<>
struct xmlApi<automaton::NPDA> {
	static automaton::NPDA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::NPDA& data);
};

template<>
struct xmlApi<automaton::SinglePopNPDA> {
	static automaton::SinglePopNPDA parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::SinglePopNPDA& data);
};

template<>
struct xmlApi<automaton::OneTapeDTM> {
	static automaton::OneTapeDTM parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::OneTapeDTM& data);
};

template<>
struct xmlApi<automaton::State> {
	static automaton::State parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const automaton::State& data);
};

template<>
struct xmlApi<grammar::Grammar> {
	static grammar::Grammar parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::Grammar& data);
};

template<>
struct xmlApi<grammar::LeftLG> {
	static grammar::LeftLG parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::LeftLG& data);
};

template<>
struct xmlApi<grammar::LeftRG> {
	static grammar::LeftRG parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::LeftRG& data);
};

template<>
struct xmlApi<grammar::RightLG> {
	static grammar::RightLG parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::RightLG& data);
};

template<>
struct xmlApi<grammar::RightRG> {
	static grammar::RightRG parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::RightRG& data);
};

template<>
struct xmlApi<grammar::LG> {
	static grammar::LG parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::LG& data);
};

template<>
struct xmlApi<grammar::CFG> {
	static grammar::CFG parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::CFG& data);
};

template<>
struct xmlApi<grammar::EpsilonFreeCFG> {
	static grammar::EpsilonFreeCFG parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::EpsilonFreeCFG& data);
};

template<>
struct xmlApi<grammar::CNF> {
	static grammar::CNF parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::CNF& data);
};

template<>
struct xmlApi<grammar::GNF> {
	static grammar::GNF parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::GNF& data);
};

template<>
struct xmlApi<grammar::CSG> {
	static grammar::CSG parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::CSG& data);
};

template<>
struct xmlApi<grammar::NonContractingGrammar> {
	static grammar::NonContractingGrammar parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::NonContractingGrammar& data);
};

template<>
struct xmlApi<grammar::ContextPreservingUnrestrictedGrammar> {
	static grammar::ContextPreservingUnrestrictedGrammar parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::ContextPreservingUnrestrictedGrammar& data);
};

template<>
struct xmlApi<grammar::UnrestrictedGrammar> {
	static grammar::UnrestrictedGrammar parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const grammar::UnrestrictedGrammar& data);
};


template<>
struct xmlApi<graph::Graph> {
	static graph::Graph parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const graph::Graph& data);
};

template<>
struct xmlApi<graph::DirectedGraph> {
	static graph::DirectedGraph parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const graph::DirectedGraph& data);
};

template<>
struct xmlApi<graph::UndirectedGraph> {
	static graph::UndirectedGraph parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const graph::UndirectedGraph& data);
};


template<>
struct xmlApi<label::Label> {
	static label::Label parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const label::Label& data);
};

template<>
struct xmlApi<label::PrimitiveLabel> {
	static label::PrimitiveLabel parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const label::PrimitiveLabel& data);
};

template<>
struct xmlApi<label::HexavigesimalLabel> {
	static label::HexavigesimalLabel parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const label::HexavigesimalLabel& data);
};

template<>
struct xmlApi<label::ObjectLabel> {
	static label::ObjectLabel parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const label::ObjectLabel& data);
};

template<>
struct xmlApi<label::LabelSetLabel> {
	static label::LabelSetLabel parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const label::LabelSetLabel& data);
};

template<>
struct xmlApi<label::LabelPairLabel> {
	static label::LabelPairLabel parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const label::LabelPairLabel& data);
};

template<>
struct xmlApi<label::UniqueLabel> {
	static label::UniqueLabel parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const label::UniqueLabel& data);
};


template<>
struct xmlApi<regexp::RegExp> {
	static regexp::RegExp parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const regexp::RegExp& data);
};

template<>
struct xmlApi<regexp::UnboundedRegExp> {
	static regexp::UnboundedRegExp parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const regexp::UnboundedRegExp& data);
};

template<>
struct xmlApi<regexp::FormalRegExp> {
	static regexp::FormalRegExp parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const regexp::FormalRegExp& data);
};


template<>
struct xmlApi<string::String> {
	static string::String parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const string::String& data);
};

template<>
struct xmlApi<string::LinearString> {
	static string::LinearString parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const string::LinearString& data);
};

template<>
struct xmlApi<string::CyclicString> {
	static string::CyclicString parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const string::CyclicString& data);
};

template<>
struct xmlApi<string::Epsilon> {
	static string::Epsilon parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const string::Epsilon& data);
};


template<>
struct xmlApi<primitive::Primitive> {
	static primitive::Primitive parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const primitive::Primitive& data);
};

template<>
struct xmlApi<primitive::String> {
	static primitive::String parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const primitive::String& data);
};

template<>
struct xmlApi<std::string> {
	static std::string parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const std::string& data);
};

template<>
struct xmlApi<primitive::Integer> {
	static primitive::Integer parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const primitive::Integer& data);
};

template<>
struct xmlApi<int> {
	static int parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, int data);
};

template<>
struct xmlApi<primitive::Character> {
	static primitive::Character parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const primitive::Character& data);
};

template<>
struct xmlApi<char> {
	static char parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, char data);
};

template<>
struct xmlApi<primitive::Unsigned> {
	static primitive::Unsigned parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const primitive::Unsigned& data);
};

template<>
struct xmlApi<unsigned> {
	static unsigned parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, unsigned data);
};

template<>
struct xmlApi<primitive::Bool> {
	static primitive::Bool parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const primitive::Bool& data);
};

template<>
struct xmlApi<bool> {
	static bool parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, bool data);
};

} /* namespace alib */

#include "container/ContainerFromXMLParser.hpp"

#include "container/ContainerToXMLComposer.hpp"

namespace alib {

template<>
struct xmlApi<container::Container> {
	static container::Container parse(std::list<sax::Token>& input);
	static bool first(const std::list<sax::Token>& input);
	static void compose(std::list<sax::Token>& output, const container::Container& data);
};

/**
 * Basic exception from which are derived all other exceptions.
 * Contains reason why the exception occured.
 */
class FromXMLParsers {
public:
	static const label::LabelFromXMLParser labelParser;
	static const alphabet::SymbolFromXMLParser symbolParser;
	static const regexp::RegExpFromXMLParser regexpParser;
	static const string::StringFromXMLParser stringParser;
	static const automaton::AutomatonFromXMLParser automatonParser;
	static const grammar::GrammarFromXMLParser grammarParser;
	static const graph::GraphFromXMLParser graphParser;
	static const exception::ExceptionFromXMLParser exceptionParser;
	static const ObjectFromXMLParser objectParser;
	static const container::ContainerFromXMLParser containerParser;
	static const primitive::PrimitiveFromXMLParser primitiveParser;

};

class ToXMLComposers : public VisitableObjectBase::const_visitor_type {
	void Visit(void*, const Void& voidObject) const;

	void Visit(void*, const exception::AlibException& symbol) const;

	void Visit(void*, const alphabet::LabeledSymbol& symbol) const;
	void Visit(void*, const alphabet::BlankSymbol& symbol) const;
	void Visit(void*, const alphabet::BottomOfTheStackSymbol& symbol) const;
	void Visit(void*, const alphabet::EndSymbol& symbol) const;
	void Visit(void*, const alphabet::RankedSymbol& symbol) const;
	void Visit(void*, const alphabet::BarSymbol& symbol) const;
	void Visit(void*, const alphabet::RankedBarSymbol& symbol) const;
	void Visit(void*, const alphabet::SubtreeWildcardSymbol& symbol) const;
	void Visit(void*, const alphabet::SymbolSetSymbol& label) const;
	void Visit(void*, const alphabet::SymbolPairSymbol& label) const;
	void Visit(void*, const alphabet::UniqueSymbol& label) const;
	void Visit(void*, const alphabet::StartSymbol& symbol) const;

	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::MultiInitialStateNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;
	void Visit(void*, const automaton::DPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenNPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const automaton::NPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void*, const automaton::OneTapeDTM& automaton) const;

	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	void Visit(void*, const graph::DirectedGraph& graph) const;
	void Visit(void*, const graph::UndirectedGraph& graph) const;

	void Visit(void*, const label::PrimitiveLabel& label) const;
	void Visit(void*, const label::HexavigesimalLabel& label) const;
	void Visit(void*, const label::ObjectLabel& label) const;
	void Visit(void*, const label::LabelSetLabel& label) const;
	void Visit(void*, const label::LabelPairLabel& label) const;
	void Visit(void*, const label::UniqueLabel& label) const;

	void Visit(void*, const regexp::UnboundedRegExp& regexp) const;
	void Visit(void*, const regexp::FormalRegExp& regexp) const;

	void Visit(void*, const string::LinearString& string) const;
	void Visit(void*, const string::CyclicString& string) const;
	void Visit(void*, const string::Epsilon& string) const;

	void Visit(void*, const container::ObjectsSet& set) const;
	void Visit(void*, const container::ObjectsVector& set) const;
	void Visit(void*, const container::ObjectsPair& set) const;
	void Visit(void*, const container::ObjectsMap& set) const;

	void Visit(void*, const primitive::String& primitive) const;
	void Visit(void*, const primitive::Integer& primitive) const;
	void Visit(void*, const primitive::Character& primitive) const;
	void Visit(void*, const primitive::Unsigned& primitive) const;
	void Visit(void*, const primitive::Bool& primitive) const;

public:
	static const label::LabelToXMLComposer labelComposer;
	static const alphabet::SymbolToXMLComposer symbolComposer;
	static const regexp::RegExpToXMLComposer regexpComposer;
	static const string::StringToXMLComposer stringComposer;
	static const automaton::AutomatonToXMLComposer automatonComposer;
	static const grammar::GrammarToXMLComposer grammarComposer;
	static const graph::GraphToXMLComposer graphComposer;
	static const exception::ExceptionToXMLComposer exceptionComposer;
	static const ObjectToXMLComposer objectComposer;
	static const container::ContainerToXMLComposer containerComposer;
	static const primitive::PrimitiveToXMLComposer primitiveComposer;

	static const ToXMLComposers toXMLComposers;
};

template<typename T>
std::set<T> xmlApi<std::set<T>>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseSet<T>(input);
}

template<typename T>
bool xmlApi<std::set<T>>::first(const std::list<sax::Token>& input) {
	return input.front().getType() == sax::Token::TokenType::START_ELEMENT && input.front().getData() == Names::CONTAINER_OBJECTS_SET;
}

template<typename T>
void xmlApi<std::set<T>>::compose(std::list<sax::Token>& output, const std::set<T>& input) {
	return ToXMLComposers::containerComposer.compose<T>(output, input);
}

template<typename T>
std::vector<T> xmlApi<std::vector<T>>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseVector<T>(input);
}

template<typename T>
bool xmlApi<std::vector<T>>::first(const std::list<sax::Token>& input) {
	return input.front().getType() == sax::Token::TokenType::START_ELEMENT && input.front().getData() == Names::CONTAINER_OBJECTS_VECTOR;
}

template<typename T>
void xmlApi<std::vector<T>>::compose(std::list<sax::Token>& output, const std::vector<T>& input) {
	return ToXMLComposers::containerComposer.compose<T>(output, input);
}

template<typename T, typename R>
std::pair<T, R> xmlApi<std::pair<T, R>>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parsePair<T, R>(input);
}

template<typename T, typename R>
bool xmlApi<std::pair<T, R>>::first(const std::list<sax::Token>& input) {
	return input.front().getType() == sax::Token::TokenType::START_ELEMENT && input.front().getData() == Names::CONTAINER_OBJECTS_PAIR;
}

template<typename T, typename R>
void xmlApi<std::pair<T, R>>::compose(std::list<sax::Token>& output, const std::pair<T, R>& input) {
	return ToXMLComposers::containerComposer.compose<T, R>(output, input);
}

template<typename T, typename R>
std::map<T, R> xmlApi<std::map<T, R>>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseMap<T, R>(input);
}

template<typename T, typename R>
bool xmlApi<std::map<T, R>>::first(const std::list<sax::Token>& input) {
	return input.front().getType() == sax::Token::TokenType::START_ELEMENT && input.front().getData() == Names::CONTAINER_OBJECTS_MAP;
}

template<typename T, typename R>
void xmlApi<std::map<T, R>>::compose(std::list<sax::Token>& output, const std::map<T, R>& input) {
	return ToXMLComposers::containerComposer.compose<T, R>(output, input);
}

/* TODO For gcc-4.9 uncomment
template<typename ... Ts>
std::variant<Ts...> xmlApi<std::variant<Ts...>>::parse(std::list<sax::Token>& input) {
	return FromXMLParsers::containerParser.parseVariant<Ts...>(input);
}

template<typename ... Ts>
bool xmlApi<std::variant<Ts...>>::first(const std::list<sax::Token>& input) {
	return input.front().getType() == sax::Token::TokenType::START_ELEMENT && input.front().getData() == Names::CONTAINER_OBJECTS_VARIANT;
}

template<typename ... Ts>
std::list<sax::Token> xmlApi<std::variant<Ts...>>::compose(std::list<sax::Token>& output, const std::variant<Ts...>& input) {
	return ToXMLComposers::containerComposer.compose<Ts...>(input);
}
*/

} /* namespace alib */

#endif /* FROM_XML_PARSERS_HPP_ */
