/*
 * LabeledSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Martin Zak
 */

#include "LabeledSymbol.h"

namespace alphabet {

LabeledSymbol::LabeledSymbol(int number) : label(label::labelFrom(number)) {

}

LabeledSymbol::LabeledSymbol(char character) : label(label::labelFrom(character)) {

}

LabeledSymbol::LabeledSymbol(const std::string& label) : label(label::labelFrom(label)) {

}

LabeledSymbol::LabeledSymbol(const label::Label& label) : label(label) {

}

LabeledSymbol::LabeledSymbol(label::Label&& label) : label(std::move(label)) {

}

SymbolBase* LabeledSymbol::clone() const {
	return new LabeledSymbol(*this);
}

SymbolBase* LabeledSymbol::plunder() && {
	return new LabeledSymbol(std::move(*this));
}

const label::Label& LabeledSymbol::getLabel() const {
	return label;
}

int LabeledSymbol::compare(const LabeledSymbol& other) const {
	return this->label.getData().compare(other.label.getData());
}

void LabeledSymbol::operator>>(std::ostream& out) const {
	out << "(LabeledSymbol " << this->label << ")";
}

LabeledSymbol::operator std::string () const {
	return (std::string) label;
}

} /* namespace alphabet */

