/*
 * NextSymbol.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "NextSymbol.h"
#include "../label/NextLabel.h"
#include "../primitive/NextPrimitive.h"
#include "LabeledSymbol.h"
#include "RankedSymbol.h"
#include "RankedBarSymbol.h"
#include "SymbolPairSymbol.h"
#include "SymbolSetSymbol.h"
#include "UniqueSymbol.h"
#include "BlankSymbol.h"
#include "BottomOfTheStackSymbol.h"
#include "EndSymbol.h"
#include "StartSymbol.h"
#include "BarSymbol.h"
#include "SubtreeWildcardSymbol.h"

namespace alphabet {

void NextSymbol::Visit(void* userData, const LabeledSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	label::NextLabel next;
	out = new Symbol(LabeledSymbol(next.nextLabel(symbol.getLabel())));
}

void NextSymbol::Visit(void* userData, const BlankSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::Integer newId(0);
	out = new Symbol(UniqueSymbol(Symbol(symbol), newId));
}

void NextSymbol::Visit(void* userData, const BottomOfTheStackSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::Integer newId(0);
	out = new Symbol(UniqueSymbol(Symbol(symbol), newId));
}

void NextSymbol::Visit(void* userData, const EndSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::Integer newId(0);
	out = new Symbol(UniqueSymbol(Symbol(symbol), newId));
}

void NextSymbol::Visit(void* userData, const StartSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::Integer newId(0);
	out = new Symbol(UniqueSymbol(Symbol(symbol), newId));
}

void NextSymbol::Visit(void* userData, const RankedSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::Integer newId(0);
	out = new Symbol(UniqueSymbol(Symbol(symbol), newId));
}

void NextSymbol::Visit(void* userData, const BarSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::Integer newId(0);
	out = new Symbol(UniqueSymbol(Symbol(symbol), newId));
}

void NextSymbol::Visit(void* userData, const RankedBarSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::Integer newId(0);
	out = new Symbol(UniqueSymbol(Symbol(symbol), newId));
}

void NextSymbol::Visit(void* userData, const SubtreeWildcardSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::Integer newId(0);
	out = new Symbol(UniqueSymbol(Symbol(symbol), newId));
}

void NextSymbol::Visit(void* userData, const SymbolPairSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::Integer newId(0);
	out = new Symbol(UniqueSymbol(Symbol(symbol), newId));
}

void NextSymbol::Visit(void* userData, const SymbolSetSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::Integer newId(0);
	out = new Symbol(UniqueSymbol(Symbol(symbol), newId));
}

void NextSymbol::Visit(void* userData, const UniqueSymbol& symbol) {
	Symbol* &out = *((Symbol**) userData);

	primitive::NextPrimitive next;
	primitive::Integer newId(symbol.getId().getData() + 1);
	out = new Symbol(UniqueSymbol(symbol.getSymbol(), newId));
}

Symbol NextSymbol::nextSymbol(const Symbol& symbol) {
	Symbol* out;
	symbol.getData().Accept((void*) &out, *this);

	Symbol nextSymbol(std::move(*out));
	delete out;
	return std::move(nextSymbol);
}

} /* namespace alphabet */
