/*
 * SubtreeWildcardSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef SUBTREE_WILDCARD_SYMBOL_H_
#define SUBTREE_WILDCARD_SYMBOL_H_

#include "Symbol.h"

namespace alphabet {

/**
 * Represents subtreeWildcard symbol for tree linearization.
 */
class SubtreeWildcardSymbol : public std::acceptor<SubtreeWildcardSymbol, VisitableSymbolBase, std::acceptor<SubtreeWildcardSymbol, alib::VisitableObjectBase, SymbolBase> > {
public:
	/**
	 * Creates a subtreeWildcard symbol.
	 * @param symbol name of the symbol
	 */
	explicit SubtreeWildcardSymbol();

	virtual SymbolBase* clone() const;
	virtual SymbolBase* plunder() &&;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const SubtreeWildcardSymbol& other) const;

	virtual void operator>>(std::ostream& out) const;

	virtual explicit operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<SubtreeWildcardSymbol>();
	}

	static SubtreeWildcardSymbol SUBTREE_WILDCARD;
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::SubtreeWildcardSymbol> {
	int operator()(const alphabet::SubtreeWildcardSymbol& first, const alphabet::SubtreeWildcardSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* SUBTREE_WILDCARD_SYMBOL_H_ */
