/*
 * Symbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_H_
#define SYMBOL_H_

#include "../std/visitor.hpp"
#include "../std/compare.hpp"
#include "SymbolBase.h"
#include "../common/wrapper.hpp"

#include <set>
#include <string>

namespace alphabet {

/**
 * Wrapper around automata.
 */
typedef alib::wrapper<SymbolBase> Symbol;

/**
 * Creates and adds unique state to grammar. If given state name is
 * already used, appends apostrophe or integer suffix
 * @param name name of the state
 * @throws AutomatonException if symbol could not be created
 * @return created symbol
 */
alphabet::Symbol createUniqueSymbol(const alphabet::Symbol& base, const std::set<alphabet::Symbol>& terminals, const std::set<alphabet::Symbol>& nonterminals);

alphabet::Symbol symbolFrom(int number);
alphabet::Symbol symbolFrom(char character);
alphabet::Symbol symbolFrom(const std::string& string);
alphabet::Symbol symbolFrom(const char* string);

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::Symbol> {
	int operator()(const alphabet::Symbol& first, const alphabet::Symbol& second) const {
		return first.getData().compare(second.getData());
	}
};

} /* namespace std */

#endif /* SYMBOL_H_ */

