/*
 * SymbolFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SymbolFromStringParser.h"
#include "../exception/AlibException.h"
#include "BlankSymbol.h"
#include "BottomOfTheStackSymbol.h"
#include "EndSymbol.h"
#include "StartSymbol.h"
#include "LabeledSymbol.h"

#include "../label/Label.h"

#include "../StringApi.hpp"

namespace alphabet {

Symbol SymbolFromStringParser::parseSymbol(std::istream& input) const {
	return parseSymbol(input, std::set<FEATURES>({FEATURES::LABELED, FEATURES::BLANK, FEATURES::BOTTOM, FEATURES::END, FEATURES::BAR, FEATURES::SUBTREE_WILDCARD, FEATURES::START}));
}

Symbol SymbolFromStringParser::parseSymbol(std::istream& input, const std::set<FEATURES>& features) const {
	SymbolFromStringLexer::Token token = m_SymbolLexer.next(input);
	switch(token.type) {
	case SymbolFromStringLexer::TokenType::BLANK:
		if(!features.count(FEATURES::BLANK)) throw exception::AlibException();
		return Symbol(BlankSymbol());
	case SymbolFromStringLexer::TokenType::BOTTOM:
		if(!features.count(FEATURES::BOTTOM)) throw exception::AlibException();
		return Symbol(BottomOfTheStackSymbol());
	case SymbolFromStringLexer::TokenType::END:
		if(!features.count(FEATURES::END)) throw exception::AlibException();
		return Symbol(EndSymbol());
	case SymbolFromStringLexer::TokenType::START:
		if(!features.count(FEATURES::START)) throw exception::AlibException();
		return Symbol(StartSymbol());
	case SymbolFromStringLexer::TokenType::ERROR:
		if(!features.count(FEATURES::LABELED)) throw exception::AlibException();
		m_SymbolLexer.putback(input, token);
		return Symbol(LabeledSymbol(alib::stringApi<label::Label>::parse(input)));
	case SymbolFromStringLexer::TokenType::TEOF:
		throw exception::AlibException("Unexpected end of file");
	}
	throw exception::AlibException();
}

} /* namespace alphabet */
