/*
 * SymbolFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SymbolFromXMLParser.h"

#include "../sax/ParserException.h"
#include "../label/Label.h"

#include "../XmlApi.hpp"

namespace alphabet {

Symbol SymbolFromXMLParser::parseSymbol(std::list<sax::Token>& input) const {
	return parseSymbol(input, std::set<FEATURES>({FEATURES::LABELED, FEATURES::BLANK, FEATURES::BOTTOM, FEATURES::END, FEATURES::RANKED, FEATURES::BAR, FEATURES::RANKED_BAR, FEATURES::SUBTREE_WILDCARD, FEATURES::SYMBOL_PAIR_SYMBOL, FEATURES::SYMBOL_SET_SYMBOL, FEATURES::UNIQUE_SYMBOL, FEATURES::START}));
}

Symbol SymbolFromXMLParser::parseSymbol(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(alib::xmlApi<LabeledSymbol>::first(input)) {
		if(!features.count(FEATURES::LABELED)) throw exception::AlibException();
		return Symbol(parseLabeledSymbol(input));
	} else if(alib::xmlApi<BlankSymbol>::first(input)) {
		if(!features.count(FEATURES::BLANK)) throw exception::AlibException();
		return Symbol(parseBlankSymbol(input));
	} else if(alib::xmlApi<BottomOfTheStackSymbol>::first(input)) {
		if(!features.count(FEATURES::BOTTOM)) throw exception::AlibException();
		return Symbol(parseBottomOfTheStackSymbol(input));
	} else if(alib::xmlApi<EndSymbol>::first(input)) {
		if(!features.count(FEATURES::END)) throw exception::AlibException();
		return Symbol(parseEndSymbol(input));
	} else if(alib::xmlApi<StartSymbol>::first(input)) {
		if(!features.count(FEATURES::START)) throw exception::AlibException();
		return Symbol(parseStartSymbol(input));
	} else if(alib::xmlApi<RankedSymbol>::first(input)) {
		if(!features.count(FEATURES::RANKED)) throw exception::AlibException();
		return Symbol(parseRankedSymbol(input));
	} else if(alib::xmlApi<BarSymbol>::first(input)) {
		if(!features.count(FEATURES::BAR)) throw exception::AlibException();
		return Symbol(parseBarSymbol(input));
	} else if(alib::xmlApi<RankedBarSymbol>::first(input)) {
		if(!features.count(FEATURES::RANKED_BAR)) throw exception::AlibException();
		return Symbol(parseRankedBarSymbol(input));
	} else if(alib::xmlApi<SubtreeWildcardSymbol>::first(input)) {
		if(!features.count(FEATURES::SUBTREE_WILDCARD)) throw exception::AlibException();
		return Symbol(parseSubtreeWildcardSymbol(input));
	} else if(alib::xmlApi<SymbolPairSymbol>::first(input)) {
		if(!features.count(FEATURES::SYMBOL_PAIR_SYMBOL)) throw exception::AlibException();
		return Symbol(parseSymbolPairSymbol(input));
	} else if(alib::xmlApi<SymbolSetSymbol>::first(input)) {
		if(!features.count(FEATURES::SYMBOL_SET_SYMBOL)) throw exception::AlibException();
		return Symbol(parseSymbolSetSymbol(input));
	} else if(alib::xmlApi<UniqueSymbol>::first(input)) {
		if(!features.count(FEATURES::UNIQUE_SYMBOL)) throw exception::AlibException();
		return Symbol(parseUniqueSymbol(input));
	} else {
		throw sax::ParserException(sax::Token("LabeledSymbol, BlankSymbol, BottomOfTheStackSymbol, EndSymbol, StartSymbol, RankedSymbol, BarSymbol, RankedBarSymbol, SubtreeWildcardSymbol, SymbolPairSymbol, SymbolSetSymbol, UniqueSymbol", sax::Token::TokenType::START_ELEMENT), input.front());
	}
}

bool SymbolFromXMLParser::first(const std::list<sax::Token>& input) const {
	if(alib::xmlApi<LabeledSymbol>::first(input) || alib::xmlApi<BlankSymbol>::first(input) || alib::xmlApi<BottomOfTheStackSymbol>::first(input) || alib::xmlApi<EndSymbol>::first(input) || alib::xmlApi<StartSymbol>::first(input) || alib::xmlApi<RankedSymbol>::first(input) || alib::xmlApi<BarSymbol>::first(input) || alib::xmlApi<RankedBarSymbol>::first(input) || alib::xmlApi<SubtreeWildcardSymbol>::first(input) || alib::xmlApi<SymbolPairSymbol>::first(input) || alib::xmlApi<SymbolSetSymbol>::first(input)) {
		return true;
	} else {
		return false;
	}
}

LabeledSymbol SymbolFromXMLParser::parseLabeledSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_LABELED_SYMBOL);
	label::Label data = alib::xmlApi<label::Label>::parse(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_LABELED_SYMBOL);
	return LabeledSymbol(data);
}

BlankSymbol SymbolFromXMLParser::parseBlankSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_BLANK_SYMBOL);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_BLANK_SYMBOL);
	return BlankSymbol();
}

BottomOfTheStackSymbol SymbolFromXMLParser::parseBottomOfTheStackSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_BOTTOM_OF_THE_STACK_SYMBOL);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_BOTTOM_OF_THE_STACK_SYMBOL);
	return BottomOfTheStackSymbol();
}

EndSymbol SymbolFromXMLParser::parseEndSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_END_SYMBOL);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_END_SYMBOL);
	return EndSymbol();
}

StartSymbol SymbolFromXMLParser::parseStartSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_END_SYMBOL);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_END_SYMBOL);
	return StartSymbol();
}

RankedSymbol SymbolFromXMLParser::parseRankedSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_RANKED_SYMBOL);
	label::Label data = alib::xmlApi<label::Label>::parse(input);
	primitive::Integer rank = alib::xmlApi<primitive::Integer>::parse(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_RANKED_SYMBOL);
	return RankedSymbol(data, rank);
}

BarSymbol SymbolFromXMLParser::parseBarSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_START_SYMBOL);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_START_SYMBOL);
	return BarSymbol();
}

RankedBarSymbol SymbolFromXMLParser::parseRankedBarSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_RANKED_BAR_SYMBOL);
	primitive::Integer rank = alib::xmlApi<primitive::Integer>::parse(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_RANKED_BAR_SYMBOL);
	return RankedBarSymbol(rank);
}

SubtreeWildcardSymbol SymbolFromXMLParser::parseSubtreeWildcardSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_SUBTREE_WILDCARD_SYMBOL);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_SUBTREE_WILDCARD_SYMBOL);
	return SubtreeWildcardSymbol();
}

SymbolSetSymbol SymbolFromXMLParser::parseSymbolSetSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_SYMBOL_SET_SYMBOL);
	std::set<Symbol> labels;
	while(isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		labels.insert(parseSymbol(input));
	}
	SymbolSetSymbol data(labels);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_SYMBOL_SET_SYMBOL);
	return data;
}

SymbolPairSymbol SymbolFromXMLParser::parseSymbolPairSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_SYMBOL_PAIR_SYMBOL);

	Symbol firstSymbol = parseSymbol(input);
	Symbol secondSymbol = parseSymbol(input);

	SymbolPairSymbol data(std::make_pair(firstSymbol, secondSymbol));
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_SYMBOL_PAIR_SYMBOL);
	return data;
}

UniqueSymbol SymbolFromXMLParser::parseUniqueSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, alib::Names::ALPHABET_UNIQUE_SYMBOL);
	
	Symbol firstSymbol = parseSymbol(input);
	primitive::Integer secondSymbol = alib::xmlApi<primitive::Integer>::parse(input);

	UniqueSymbol data(firstSymbol, secondSymbol);
	popToken(input, sax::Token::TokenType::END_ELEMENT, alib::Names::ALPHABET_UNIQUE_SYMBOL);
	return data;
}

} /* namespace alphabet */
