/*
 * SymbolFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_FROM_XML_PARSER_H_
#define SYMBOL_FROM_XML_PARSER_H_

#include "../sax/FromXMLParserHelper.h"
#include <vector>
#include <set>
#include "Symbol.h"
#include "SymbolFeatures.h"
#include "../sax/Token.h"
#include "../label/Label.h"

#include "BlankSymbol.h"
#include "BottomOfTheStackSymbol.h"
#include "EndSymbol.h"
#include "StartSymbol.h"
#include "LabeledSymbol.h"
#include "RankedSymbol.h"
#include "BarSymbol.h"
#include "RankedBarSymbol.h"
#include "SubtreeWildcardSymbol.h"
#include "SymbolPairSymbol.h"
#include "SymbolSetSymbol.h"
#include "UniqueSymbol.h"

namespace alib {

template<typename T>
struct xmlApi;

} /* namespace alib */

namespace alphabet {

/**
 * Parser used to get String from XML parsed into list of tokens.
 */
class SymbolFromXMLParser : public sax::FromXMLParserHelper {
	Symbol parseSymbol(std::list<sax::Token>& input) const;
	Symbol parseSymbol(std::list<sax::Token>& input, const std::set<FEATURES>& features) const;

	LabeledSymbol parseLabeledSymbol(std::list<sax::Token>& input) const;
	BlankSymbol parseBlankSymbol(std::list<sax::Token>& input) const;
	BottomOfTheStackSymbol parseBottomOfTheStackSymbol(std::list<sax::Token>& input) const;
	EndSymbol parseEndSymbol(std::list<sax::Token>& input) const;
	StartSymbol parseStartSymbol(std::list<sax::Token>& input) const;
	RankedSymbol parseRankedSymbol(std::list<sax::Token>& input) const;
	BarSymbol parseBarSymbol(std::list<sax::Token>& input) const;
	RankedBarSymbol parseRankedBarSymbol(std::list<sax::Token>& input) const;
	SubtreeWildcardSymbol parseSubtreeWildcardSymbol(std::list<sax::Token>& input) const;
	SymbolPairSymbol parseSymbolPairSymbol(std::list<sax::Token>& input) const;
	SymbolSetSymbol parseSymbolSetSymbol(std::list<sax::Token>& input) const;
	UniqueSymbol parseUniqueSymbol(std::list<sax::Token>& input) const;

	template<typename T> friend class alib::xmlApi;
public:
	bool first(const std::list<sax::Token>& input) const;
};

} /* namespace alphabet */

#endif /* SYMBOL_FROM_XML_PARSER_H_ */
