/*
 * SymbolPairSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "SymbolPairSymbol.h"
#include "../std/pair.hpp"
#include <sstream>

namespace alphabet {

SymbolPairSymbol::SymbolPairSymbol(const std::pair<Symbol, Symbol>& symbol) : symbol(std::move(symbol)) {

}

SymbolPairSymbol::SymbolPairSymbol(std::pair<Symbol, Symbol>&& symbol) : symbol(std::move(symbol)) {

}

SymbolBase* SymbolPairSymbol::clone() const {
	return new SymbolPairSymbol(*this);
}

SymbolBase* SymbolPairSymbol::plunder() && {
	return new SymbolPairSymbol(std::move(*this));
}

const std::pair<Symbol, Symbol>& SymbolPairSymbol::getData() const {
	return symbol;
}

int SymbolPairSymbol::compare(const SymbolPairSymbol& other) const {
	int res = symbol.first.getData().compare(other.symbol.first.getData());
	if(res == 0) res = symbol.second.getData().compare(other.symbol.second.getData());
	return res;
}

void SymbolPairSymbol::operator>>(std::ostream& out) const {
	out << "(SymbolPairSymbol " << symbol << ")";
}

SymbolPairSymbol::operator std::string() const {
	std::stringstream ss;
	ss << "<";
	ss << (std::string) symbol.first;
	ss << ",";
	ss << (std::string) symbol.second;
	ss << "}";
	return std::move(ss).str();
}

} /* namespace alphabet */

