/*
 * SymbolToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SymbolToStringComposer.h"
#include "LabeledSymbol.h"
#include "RankedSymbol.h"
#include "RankedBarSymbol.h"
#include "SymbolPairSymbol.h"
#include "SymbolSetSymbol.h"
#include "UniqueSymbol.h"

#include "../StringApi.hpp"

namespace alphabet {

void SymbolToStringComposer::Visit(void* userData, const LabeledSymbol& symbol) const {
	std::ostream &out = *((std::ostream*) userData);

	alib::stringApi<label::Label>::compose(out, symbol.getLabel());
}

void SymbolToStringComposer::Visit(void* userData, const BlankSymbol&) const {
	std::ostream &out = *((std::ostream*) userData);

	out << "#B";
}

void SymbolToStringComposer::Visit(void* userData, const BottomOfTheStackSymbol&) const {
	std::ostream &out = *((std::ostream*) userData);

	out << "#T";
}

void SymbolToStringComposer::Visit(void* userData, const EndSymbol&) const {
	std::ostream &out = *((std::ostream*) userData);

	out << "#$";
}

void SymbolToStringComposer::Visit(void* userData, const StartSymbol&) const {
	std::ostream &out = *((std::ostream*) userData);

	out << "#S";
}

void SymbolToStringComposer::Visit(void* userData, const RankedSymbol& symbol) const {
	std::ostream &out = *((std::ostream*) userData);

	alib::stringApi<label::Label>::compose(out, symbol.getLabel());
	alib::stringApi<primitive::Primitive>::compose(out, primitive::Primitive(symbol.getRank()));
}

void SymbolToStringComposer::Visit(void* userData, const BarSymbol&) const {
	std::ostream &out = *((std::ostream*) userData);

	out << "#|";
}

void SymbolToStringComposer::Visit(void* userData, const RankedBarSymbol& symbol) const {
	std::ostream &out = *((std::ostream*) userData);

	out << "#|";
	alib::stringApi<primitive::Primitive>::compose(out, primitive::Primitive(symbol.getRank()));
}

void SymbolToStringComposer::Visit(void* userData, const SubtreeWildcardSymbol&) const {
	std::ostream &out = *((std::ostream*) userData);

	out << "#S";
}

void SymbolToStringComposer::Visit(void* userData, const SymbolSetSymbol& symbol) const {
	std::ostream &out = *((std::ostream*) userData);

	out << '[';
	bool first = true;
	for(const Symbol& innerSymbol : symbol.getData()) {
		if(!first)
			out << ", ";
		else
			first = false;
		innerSymbol.getData().Accept(userData, *this);
	}
	out << ']';
}

void SymbolToStringComposer::Visit(void* userData, const SymbolPairSymbol& symbol) const {
	std::ostream &out = *((std::ostream*) userData);

	out << '<';
	symbol.getData().first.getData().Accept(userData, *this);
	out << ", ";
	symbol.getData().second.getData().Accept(userData, *this);
	out << '>';
}

void SymbolToStringComposer::Visit(void* userData, const UniqueSymbol& symbol) const {
	std::ostream &out = *((std::ostream*) userData);

	out << '<';
	symbol.getSymbol().getData().Accept(userData, *this);
	out << ", ";
	alib::stringApi<primitive::Primitive>::compose(out, primitive::Primitive(symbol.getId()));
	out << '>';
}

void SymbolToStringComposer::compose(std::ostream& out, const Symbol& symbol) const {
	symbol.getData().Accept((void*) &out, *this);
}

} /* namespace alphabet */
