/*
 * SymbolToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SymbolToXMLComposer.h"

#include "../XmlApi.hpp"

namespace alphabet {

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const Symbol& symbol) const {
	symbol.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const SymbolBase& symbol) const {
	symbol.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const LabeledSymbol& symbol) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_LABELED_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<label::Label>::compose(out, symbol.getLabel());
	out.push_back(sax::Token(alib::Names::ALPHABET_LABELED_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const BlankSymbol&) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_BLANK_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(alib::Names::ALPHABET_BLANK_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const BottomOfTheStackSymbol&) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_BOTTOM_OF_THE_STACK_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(alib::Names::ALPHABET_BOTTOM_OF_THE_STACK_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const EndSymbol&) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_END_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(alib::Names::ALPHABET_END_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const StartSymbol&) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_START_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(alib::Names::ALPHABET_START_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const RankedSymbol& symbol) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_RANKED_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<label::Label>::compose(out, symbol.getLabel());
	alib::xmlApi<primitive::Integer>::compose(out, symbol.getRank());
	out.push_back(sax::Token(alib::Names::ALPHABET_RANKED_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const BarSymbol&) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_BAR_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(alib::Names::ALPHABET_BAR_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const RankedBarSymbol& symbol) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_RANKED_BAR_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<primitive::Integer>::compose(out, symbol.getRank());
	out.push_back(sax::Token(alib::Names::ALPHABET_RANKED_BAR_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const SubtreeWildcardSymbol&) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_SUBTREE_WILDCARD_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(alib::Names::ALPHABET_SUBTREE_WILDCARD_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const SymbolSetSymbol& symbol) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_SYMBOL_SET_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	for(const Symbol& innerSymbol : symbol.getData()) {
		alib::xmlApi<alphabet::Symbol>::compose(out, innerSymbol);
	}
	out.push_back(sax::Token(alib::Names::ALPHABET_SYMBOL_SET_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const SymbolPairSymbol& symbol) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_SYMBOL_PAIR_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol.getData().first);
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol.getData().second);
	out.push_back(sax::Token(alib::Names::ALPHABET_SYMBOL_PAIR_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

void SymbolToXMLComposer::compose(std::list<sax::Token>& out, const UniqueSymbol& symbol) const {
	out.push_back(sax::Token(alib::Names::ALPHABET_UNIQUE_SYMBOL, sax::Token::TokenType::START_ELEMENT));
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol.getSymbol());
	alib::xmlApi<primitive::Integer>::compose(out, symbol.getId());
	out.push_back(sax::Token(alib::Names::ALPHABET_UNIQUE_SYMBOL, sax::Token::TokenType::END_ELEMENT));
}

} /* namespace alphabet */
