/*
 * SymbolToXMLComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnciek
 */

#ifndef SYMBOL_TO_XML_COMPOSER_H_
#define SYMBOL_TO_XML_COMPOSER_H_

#include <list>
#include "Symbol.h"
#include "../sax/Token.h"
#include "../label/Label.h"

namespace alib {

template<typename T>
struct xmlApi;

} /* namespace alib */

namespace alphabet {

/**
 * This class contains methods to print XML representation of string to the output stream.
 */
class SymbolToXMLComposer {
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	void compose(std::list<sax::Token>& output, const SymbolBase& symbol) const;

	/**
	 * Prints XML representation of String to the output stream.
	 * @param string String to print
	 * @param out output stream to which print the String
	 */
	void compose(std::list<sax::Token>& output, const Symbol& symbol) const;

	void compose(std::list<sax::Token>& output, const LabeledSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const BlankSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const BottomOfTheStackSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const EndSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const StartSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const RankedSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const BarSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const RankedBarSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const SubtreeWildcardSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const SymbolPairSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const SymbolSetSymbol& symbol) const;
	void compose(std::list<sax::Token>& output, const UniqueSymbol& symbol) const;

	template<typename T> friend class alib::xmlApi;
};

} /* namespace alphabet */

#endif /* SYMBOL_TO_XML_COMPOSER_H_ */
