/*
 * AutomatonFromStringParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_FROM_STRING_PARSER_H_
#define AUTOMATON_FROM_STRING_PARSER_H_

#include "AutomatonFromStringLexer.h"

#include "Automaton.h"
#include "AutomatonFeatures.h"
#include "common/State.h"

#include <tuple>
#include "../std/variant.hpp"
#include "../string/Epsilon.h"

namespace alib {

template<typename T>
struct stringApi;

} /* namespace alib */

namespace automaton {

class AutomatonFromStringParser {
	AutomatonFromStringLexer m_AutomatonLexer;

	void initialFinalState(std::istream& input, bool& initial, bool& final) const;

	void parseEpsilonNFATransition(std::istream& input, std::set<State>& states, const std::vector<std::variant<string::Epsilon, alphabet::Symbol>>& symbols, State*& initialState, std::set<State>& finalStates, std::set<std::tuple<State, std::variant<string::Epsilon, alphabet::Symbol>, State>>& transitionFunction) const;
	void parseMultiInitialStateNFATransition(std::istream& input, MultiInitialStateNFA& res, const std::vector<alphabet::Symbol>& symbols) const;
	void parseNFATransition(std::istream& input, std::set<State>& states, const std::vector<alphabet::Symbol>& symbols, State*& initialState, std::set<State>& finalStates, std::set<std::tuple<State, alphabet::Symbol, State>>& transitionFunction) const;
	void parseDFATransition(std::istream& input, std::set<State>& states, const std::vector<alphabet::Symbol>& symbols, State*& initialState, std::set<State>& finalStates, std::set<std::tuple<State, alphabet::Symbol, State>>& transitionFunction) const;

	Automaton parseAutomaton(std::istream& input) const;
	Automaton parseAutomaton(std::istream& input, const std::set<FEATURES>& features) const;

	EpsilonNFA parseEpsilonNFA(std::istream& input) const;
	NFA parseNFA(std::istream& input) const;
	MultiInitialStateNFA parseMultiInitialStateNFA(std::istream& input) const;
	DFA parseDFA(std::istream& input) const;

	template<typename T> friend class alib::stringApi;
};

} /* namespace automaton */

#endif /* AUTOMATON_FROM_STRING_PARSER_H_ */
