/*
 * AutomatonToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_TO_STRING_COMPOSER_H_
#define AUTOMATON_TO_STRING_COMPOSER_H_

#include <sstream>
#include "Automaton.h"
#include "FSM/DFA.h"
#include "FSM/NFA.h"
#include "FSM/MultiInitialStateNFA.h"
#include "FSM/EpsilonNFA.h"

namespace automaton {

class AutomatonToStringComposer : public VisitableAutomatonBase::const_visitor_type {
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const MultiInitialStateNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;

	void composeTransitionsFromState(std::ostream& out, const DFA& automaton, const State& from) const;
	void composeTransitionsFromState(std::ostream& out, const NFA& automaton, const State& from) const;
	void composeTransitionsFromState(std::ostream& out, const MultiInitialStateNFA& automaton, const State& from) const;
	void composeTransitionsFromState(std::ostream& out, const EpsilonNFA& automaton, const State& from) const;

public:
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	void compose(std::ostream& output, const AutomatonBase& automaton) const;

	void compose(std::ostream& output, const Automaton& automaton) const;

	void compose(std::ostream& output, const DFA& automaton) const;
	void compose(std::ostream& output, const NFA& automaton) const;
	void compose(std::ostream& output, const MultiInitialStateNFA& automaton) const;
	void compose(std::ostream& output, const EpsilonNFA& automaton) const;
};

} /* namespace automaton */

#endif /* AUTOMATON_TO_STRING_COMPOSER_H_ */
