/*
 * InputAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "InputAlphabet.h"

#include <algorithm>
#include "../AutomatonException.h"

namespace automaton {

const std::set<alphabet::Symbol>& InputAlphabet::getInputAlphabet() const {
	return inputAlphabet;
}

bool InputAlphabet::addInputSymbol(const alphabet::Symbol& symbol) {
	return inputAlphabet.insert(symbol).second;
}

void InputAlphabet::addInputSymbols(const std::set<alphabet::Symbol>& symbols) {
	return inputAlphabet.insert(symbols.begin(), symbols.end());
}

void InputAlphabet::setInputAlphabet(const std::set<alphabet::Symbol>& newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(inputAlphabet.begin(), inputAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(newSymbols.begin(), newSymbols.end(), inputAlphabet.begin(), inputAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeInputSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addInputSymbol(addedSymbol);
	}
}

} /* namespace automaton */

