/*
 * InputAutomaton.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef INPUT_ALPHABET_H_
#define INPUT_ALPHABET_H_

#include <set>
#include "../../alphabet/Symbol.h"


namespace automaton {

/**
 * Abstract base class for all automatons. Contains common elements of automatons.
 */
class InputAlphabet {
protected:
	std::set<alphabet::Symbol> inputAlphabet;
public:
	/**
	 * Adds input symbol to input alphabet.
	 * @param symbol Symbol to add
	 * @throws AutomatonException when symbol already exists
	 */
	bool addInputSymbol(const alphabet::Symbol& symbol);

	/**
	 * Adds input symbols to input alphabet.
	 * @param symbol Symbol to add
	 * @throws AutomatonException when symbol already exists
	 */
	void addInputSymbols(const std::set<alphabet::Symbol>& symbols);

	/**
	 * Sets input symbols of the automaton.
	 * @param symbols Symbols to set
	 * @throws AutomatonException when symbol already exists
	 */
	void setInputAlphabet(const std::set<alphabet::Symbol>& symbols);

	/**
	 * Removes input symbol from the input alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is not present in input alphabet
	 * or when symbol is part of the transition
	 */
	virtual bool removeInputSymbol(const alphabet::Symbol& symbol) = 0;

	/**
	 * @return the input alphabet
	 */
	const std::set<alphabet::Symbol>& getInputAlphabet() const;

};

} /* namespace automaton */

#endif /* INPUT_ALPHABET_H_ */

