/*
 * State.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Martin Zak
 */

#include "State.h"
#include "../../label/Label.h"

namespace automaton {

State::State(const label::Label& name) : name(name) {
}

State::State(label::Label&& name) : name(std::move(name)) {
}

State::State(int number) : name(label::labelFrom(number)) {
}

State::State(int number1, int number2) : name(label::labelFrom(number1, number2)) {
}

State::State(char character) : name(label::labelFrom(character)) {
}

State::State(const std::string& name) : name(label::labelFrom(name)) {
}

State::State(const char* name) : State((std::string) name) {
}

const label::Label& State::getName() const {
	return name;
}

bool State::operator < (const State& other) const {
	return this->compare(other) < 0;
}

bool State::operator == (const State& other) const {
	return this->compare(other) == 0;
}

bool State::operator != (const State& other) const{
	return !(*this == other);
}

int State::compare(const State& other) const {
	return name.getData().compare(other.name.getData());
}

std::ostream& operator<<(std::ostream& out, const State& state) {
	out << "(State " << state.name << ")";
	return out;
}

State::operator std::string () const {
	return (std::string) name;
}

} /* namespace automaton */
