/*
 * ContainerToXMLComposer.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "ContainerToXMLComposer.hpp"

namespace container {

void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const ContainerBase& container) const {
	container.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const Container& container) const {
	container.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const ObjectsSet& container) const {
	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_SET, sax::Token::TokenType::START_ELEMENT));

	for(const alib::Object& item : container) {
		alib::xmlApi<alib::Object>::compose(out, item);
	}

	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_SET, sax::Token::TokenType::END_ELEMENT));
}

void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const ObjectsVector& container) const {
	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_VECTOR, sax::Token::TokenType::START_ELEMENT));

	for(const alib::Object& item : container) {
		alib::xmlApi<alib::Object>::compose(out, item);
	}

	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_VECTOR, sax::Token::TokenType::END_ELEMENT));
}

void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const ObjectsPair& container) const {
	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_PAIR, sax::Token::TokenType::START_ELEMENT));

	alib::xmlApi<alib::Object>::compose(out, container.first);
	alib::xmlApi<alib::Object>::compose(out, container.second);

	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_PAIR, sax::Token::TokenType::END_ELEMENT));
}

void ContainerToXMLComposer::compose(std::list<sax::Token>& out, const ObjectsMap& container) const {
	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_MAP, sax::Token::TokenType::START_ELEMENT));

	for(const std::pair<alib::Object, alib::Object>& item : container) {
		alib::xmlApi<std::pair<alib::Object, alib::Object>>::compose(out, item);
	}

	out.push_back(sax::Token(alib::Names::CONTAINER_OBJECTS_MAP, sax::Token::TokenType::END_ELEMENT));
}

} /* namespace container */

