/*
 * Set.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_SET_H_
#define OBJECTS_SET_H_

#include <set>
#include <string>

#include "../object/Object.h"
#include "ContainerBase.h"

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
class ObjectsSet : public std::set<alib::Object>, public std::acceptor<ObjectsSet, VisitableContainerBase, std::acceptor<ObjectsSet, alib::VisitableObjectBase, ContainerBase> > {
public:
	virtual ContainerBase* clone() const;
	
	virtual ContainerBase* plunder() &&;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const ObjectsSet& other) const;

	virtual void operator>>(std::ostream& os) const;

	virtual explicit operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<ObjectsSet>();
	}
};

} /* namespace container */

namespace std {

template<>
struct compare<container::ObjectsSet> {
	int operator()(const container::ObjectsSet& first, const container::ObjectsSet& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* OBJECTS_SET_H_ */
