/*
 * AlibException.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#include "AlibException.h"

#include <cstdlib>
#include <iostream>
#include <stdexcept>
#include <sstream>

#include <execinfo.h>
#ifdef DEBUG
	#include "../debug/bfdStacktrace.h"
#endif

namespace exception {

AlibException::AlibException ( ) {
	#ifdef DEBUG
		this->backtrace = std::bfdStacktrace();
	#else
		this->backtrace = "";
	#endif

	this->whatMessage += this->backtrace;
}

AlibException::AlibException ( const std::string & cause ) : AlibException { } {
	this->cause = cause;
	
	this->whatMessage += this->cause;
}

AlibException::AlibException ( const std::string & cause, const std::string& backtrace ) : AlibException { } {
	this->backtrace = backtrace;
	this->cause = cause;

	this->whatMessage += this->backtrace;
	this->whatMessage += this->cause;
}

AlibException::~AlibException ( ) noexcept {

}

alib::ObjectBase* AlibException::clone() const {
	return new AlibException(*this);
}

alib::ObjectBase* AlibException::plunder() && {
	return new AlibException(std::move(*this));
}

const char * AlibException::what ( ) const noexcept {
	return whatMessage.c_str ( );
}

const std::string & AlibException::getCause ( ) const {
	return cause;
}

const std::string & AlibException::getBacktrace ( ) const {
	return backtrace;
}

int AlibException::compare(const AlibException& other) const {
	return this->whatMessage.compare(other.whatMessage);
}

void AlibException::operator>>(std::ostream& os) const {
	os << this->whatMessage;
}

AlibException::operator std::string() const {
	return this->whatMessage;
}

} /* namespace exception */
