/*
 * GrammarBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef GRAMMAR_BASE_H_
#define GRAMMAR_BASE_H_

#include "../common/base.hpp"
#include "../object/ObjectBase.h"
#include "../exception/AlibException.h"

namespace grammar {

class GrammarBase;

typedef std::acceptor_base<GrammarBase,
			grammar::LeftLG, grammar::LeftRG, grammar::RightLG, grammar::RightRG, grammar::LG, grammar::CFG, grammar::EpsilonFreeCFG, grammar::CNF, grammar::GNF, grammar::CSG, grammar::NonContractingGrammar, grammar::ContextPreservingUnrestrictedGrammar, grammar::UnrestrictedGrammar
	> VisitableGrammarBase;

/**
 * Abstract base class for all grammars.
 */
class GrammarBase : public alib::ObjectBase, public VisitableGrammarBase {
public:
	using VisitableGrammarBase::Accept;
	using alib::VisitableObjectBase::Accept;

	virtual GrammarBase* clone() const = 0;
	virtual GrammarBase* plunder() && = 0;

};

class VisitableConstRGBase : public VisitableGrammarBase::const_visitor_type {

	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar&) const
	{
	    throw exception::AlibException("Unsupported grammar type ContextPreservingUnrestrictedGrammar");
	}

	void Visit(void*, const grammar::UnrestrictedGrammar&) const
	{
	    throw exception::AlibException("Unsupported grammar type UnrestrictedGrammar");
	}

	void Visit(void*, const grammar::CSG&) const
	{
	    throw exception::AlibException("Unsupported grammar type CSG");
	}

	void Visit(void*, const grammar::NonContractingGrammar&) const
	{
	    throw exception::AlibException("Unsupported grammar type NonContractingGrammar");
	}

	void Visit(void*, const grammar::CNF&) const
	{
	    throw exception::AlibException("Unsupported grammar type CNF");
	}

	void Visit(void*, const grammar::CFG&) const
	{
	    throw exception::AlibException("Unsupported grammar type CFG");
	}

	void Visit(void*, const grammar::EpsilonFreeCFG&) const
	{
	    throw exception::AlibException("Unsupported grammar type EpsilonFreeCFG");
	}

	void Visit(void*, const grammar::GNF&) const
	{
	    throw exception::AlibException("Unsupported grammar type GNF");
	}

	void Visit(void*, const grammar::LG&) const
	{
	    throw exception::AlibException("Unsupported grammar type LG");
	}

};

} /* namespace grammar */

namespace std {

template<>
struct compare<grammar::GrammarBase> {
	int operator()(const grammar::GrammarBase& first, const grammar::GrammarBase& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* GRAMMAR_BASE_H_ */

