#include "GrammarToStringComposer.h"
#include <sstream>

#include "Regular/RightRG.h"
#include "Regular/RightLG.h"
#include "Regular/LeftRG.h"
#include "Regular/LeftLG.h"

#include "ContextFree/CFG.h"
#include "ContextFree/EpsilonFreeCFG.h"
#include "ContextFree/LG.h"
#include "ContextFree/CNF.h"
#include "ContextFree/GNF.h"

#include "ContextSensitive/CSG.h"
#include "ContextSensitive/NonContractingGrammar.h"

#include "Unrestricted/ContextPreservingUnrestrictedGrammar.h"
#include "Unrestricted/UnrestrictedGrammar.h"

#include "../StringApi.hpp"

namespace grammar {

template<class T>
void GrammarToStringComposer::composeCFLikeGrammar(std::ostream& output, const T& grammar) const {
	bool first;

	output << " (" << std::endl;

	output << "{";
	first = false;
	for(const auto& symbol : grammar.getNonterminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = false;
	for(const auto& symbol : grammar.getTerminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{ ";
	first = true;
	for(const auto& rule : grammar.getRawRules() ) {
		if(first)
			first = false;
		else
			output << "," << std::endl << "  ";
		alib::stringApi<alphabet::Symbol>::compose(output, rule.first);
		output << " ->";
		bool innerFirst = true;
		for(const auto& rhs : rule.second) {
			if(innerFirst)
				innerFirst = false;
			else
				output << " |";
			for(const auto& symbol : rhs) {
				output << " ";
				alib::stringApi<alphabet::Symbol>::compose(output, symbol);
			}
		}
	}
	output << "}," << std::endl;
	alib::stringApi<alphabet::Symbol>::compose(output, grammar.getInitialSymbol());
	output << ")" << std::endl;
}

template<class T>
void GrammarToStringComposer::composeCSLikeGrammar(std::ostream& output, const T& grammar) const {
	bool first;

	output << " (" << std::endl;

	output << "{";
	first = false;
	for(const auto& symbol : grammar.getNonterminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = false;
	for(const auto& symbol : grammar.getTerminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = true;
	for(const auto& rule : grammar.getRules() ) {
		if(first)
			first = false;
		else
			output << "," << std::endl << " ";
		for(const auto& symbol : rule.first) {
			output << " ";
			alib::stringApi<alphabet::Symbol>::compose(output, symbol);
		}
		output << " ->";
		bool innerFirst = true;
		for(const auto& rhs : rule.second) {
			if(innerFirst)
				innerFirst = false;
			else
				output << " |";
			for(const auto& symbol : rhs) {
				output << " ";
				alib::stringApi<alphabet::Symbol>::compose(output, symbol);
			}
		}
	}
	output << "}," << std::endl;
	alib::stringApi<alphabet::Symbol>::compose(output, grammar.getInitialSymbol());
	output << ")" << std::endl;
}

template<class T>
void GrammarToStringComposer::composePreservingCSLikeGrammar(std::ostream& output, const T& grammar) const {
	bool first;

	output << " (" << std::endl;

	output << "{";
	first = false;
	for(const auto& symbol : grammar.getNonterminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = false;
	for(const auto& symbol : grammar.getTerminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = true;
	for(const auto& rule : grammar.getRules() ) {
		if(first)
			first = false;
		else
			output << "," << std::endl << " ";
		for(const auto& symbol : std::get<0>(rule.first)) {
			output << " ";
			alib::stringApi<alphabet::Symbol>::compose(output, symbol);
		}
		output << " | ";
		alib::stringApi<alphabet::Symbol>::compose(output, std::get<1>(rule.first));
		output << " |";
		for(const auto& symbol : std::get<2>(rule.first)) {
			output << " ";
			alib::stringApi<alphabet::Symbol>::compose(output, symbol);
		}
		output << " ->";
		bool innerFirst = true;
		for(const auto& rhs : rule.second) {
			if(innerFirst)
				innerFirst = false;
			else
				output << " |";
			for(const auto& symbol : rhs) {
				output << " ";
				alib::stringApi<alphabet::Symbol>::compose(output, symbol);
			}
		}
	}
	output << "}," << std::endl;
	alib::stringApi<alphabet::Symbol>::compose(output, grammar.getInitialSymbol());
	output << ")" << std::endl;
}

void GrammarToStringComposer::compose(std::ostream& output, const Grammar& grammar) const {
	grammar.getData().Accept((void*) &output, *this);
}

void GrammarToStringComposer::compose(std::ostream& output, const LeftLG& grammar) const {
	output << "LEFT_LG";
	composeCFLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const LeftRG& grammar) const {
	output << "LEFT_RG";
	composeCFLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const RightLG& grammar) const {
	output << "RIGHT_LG";
	composeCFLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const RightRG& grammar) const {
	output << "RIGHT_RG";
	composeCFLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const LG& grammar) const {
	output << "LG";
	composeCFLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const CFG& grammar) const {
	output << "CFG";
	composeCFLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const EpsilonFreeCFG& grammar) const {
	output << "EPSILON_FREE_CFG";
	composeCFLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const CNF& grammar) const {
	output << "CNF";
	composeCFLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const GNF& grammar) const {
	output << "GNF";
	composeCFLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const CSG& grammar) const {
	output << "CSG";
	composePreservingCSLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const NonContractingGrammar& grammar) const {
	output << "NON_CONTRACTING_GRAMMAR";
	composeCSLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const ContextPreservingUnrestrictedGrammar& grammar) const {
	output << "CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR";
	composePreservingCSLikeGrammar(output, grammar);
}

void GrammarToStringComposer::compose(std::ostream& output, const UnrestrictedGrammar& grammar) const {
	output << "UNRESTRICTED_GRAMMAR";
	composeCSLikeGrammar(output, grammar);
}

void GrammarToStringComposer::Visit(void* data, const LeftLG& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const LeftRG& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const RightLG& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const RightRG& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const LG& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const CFG& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const EpsilonFreeCFG& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const CNF& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const GNF& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const CSG& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const NonContractingGrammar& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const ContextPreservingUnrestrictedGrammar& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

void GrammarToStringComposer::Visit(void* data, const UnrestrictedGrammar& grammar) const {
	this->compose(*((std::ostream*) data), grammar);
}

}
