#ifndef GRAPH_FROM_STRING_PARSER_H_
#define GRAPH_FROM_STRING_PARSER_H_

#include "Graph.h"
#include "GraphFeatures.h"
#include "GraphRepresentation.h"
#include "directed/DirectedGraph.h"
#include "undirected/UndirectedGraph.h"

#include "GraphFromStringLexer.h"
#include "GraphFeatures.h"

namespace alib {

template<typename T>
struct stringApi;

} // namespace alib

namespace graph {

class GraphFromStringParser
{
private:
	Graph parseGraph(std::istream &input) const;
	DirectedGraph parseDirectedGraph(std::istream &input) const;
	UndirectedGraph parseUndirectedGraph(std::istream &input) const;

	REPRESENTATION parseRepresentation(std::istream &input) const;
	Node parseNode(std::istream &input) const;
	DirectedEdge parseDirectedEdge(std::istream &input) const;
	UndirectedEdge parseUndirectedEdge(std::istream &input) const;

	template<typename T>
	void parseNodes(std::istream &input, T &graph) const;
	void parseDirectedEdges(std::istream &input, DirectedGraph &graph) const;
	void parseUndirectedEdges(std::istream &input, UndirectedGraph &graph) const;
	void parseDelimiter(std::istream &input) const;
	int parseValue(std::istream &input) const;

	template<typename T>
	void parseNodeValues(std::istream &input, T &graph) const;
	void parseDirectedEdgeValues(std::istream &input, DirectedGraph &graph) const;
	void parseUndirectedEdgeValues(std::istream &input, UndirectedGraph &graph) const;

	GraphFromStringLexer lexer;

	template<typename T> friend class alib::stringApi;
};

} // namespace graph

#endif // GRAPH_FROM_STRING_PARSER_H_
