#ifndef GRAPH_FROM_XML_PARSER_H_
#define GRAPH_FROM_XML_PARSER_H_

#include "../sax/FromXMLParserHelper.h"
#include "Graph.h"
#include "GraphFeatures.h"
#include "directed/DirectedGraph.h"
#include "undirected/UndirectedGraph.h"
#include "../sax/Token.h"
#include "../alphabet/Symbol.h"

namespace alib {

template<typename T>
struct xmlApi;

} // namespace alib

namespace graph {

// Parser used to get Graph from XML parsed into list of tokens.
class GraphFromXMLParser : public sax::FromXMLParserHelper
{
public:
	bool first(const std::list<sax::Token> &input) const;

private:
	Graph parseGraph(std::list<sax::Token> &input) const;
	DirectedGraph parseDirectedGraph(std::list<sax::Token> &input) const;
	UndirectedGraph parseUndirectedGraph(std::list<sax::Token> &input) const;

	REPRESENTATION parseRepresentation(std::list<sax::Token> &input) const;
	Node parseNode(std::list<sax::Token> &input) const;
	DirectedEdge parseDirectedEdge(std::list<sax::Token> &input) const;
	UndirectedEdge parseUndirectedEdge(std::list<sax::Token> &input) const;
	int parseValue(std::list<sax::Token> &input) const;

	template<typename T>
	void parseNodes(std::list<sax::Token> &input, T &graph) const;
	void parseDirectedEdges(std::list<sax::Token> &input, DirectedGraph &graph) const;
	void parseUndirectedEdges(std::list<sax::Token> &input, UndirectedGraph &graph) const;

	template<typename T>
	void parseNodeValues(std::list<sax::Token> &input, T &graph) const;
	void parseDirectedEdgeValues(std::list<sax::Token> &input, DirectedGraph &graph) const;
	void parseUndirectedEdgeValues(std::list<sax::Token> &input, UndirectedGraph &graph) const;

	template<typename T> friend class alib::xmlApi;
};

} // namespace graph

#endif // GRAPH_FROM_XML_PARSER_H_
