#ifndef GRAPH_TO_STRING_COMPOSER_H_
#define GRAPH_TO_STRING_COMPOSER_H_

#include <set>
#include <string>

#include "Graph.h"
#include "GraphRepresentation.h"
#include "common/GraphElement.h"
#include "directed/DirectedGraph.h"
#include "undirected/UndirectedGraph.h"
#include "../std/visitor.hpp"
#include "../sax/Token.h"

namespace graph {

class GraphToStringComposer : public VisitableGraphBase::const_visitor_type, GraphElement::const_visitor_type
{
public:
	void compose(std::ostream &out, const Graph &graph) const;

private:
	void Visit(void *data, const DirectedGraph &graph) const;
	void Visit(void *data, const UndirectedGraph &graph) const;

	void Visit(void *data, const Node &node) const;
	void Visit(void *data, const DirectedEdge &edge) const;
	void Visit(void *data, const UndirectedEdge &edge) const;

	void composeRepresentation(std::ostream &out, REPRESENTATION representation) const;
	void composeNodes(std::ostream &out, const std::set<Node> &nodes) const;
	void composeDirectedEdges(std::ostream &out, const std::set<DirectedEdge> &edges) const;
	void composeUndirectedEdges(std::ostream &out, const std::set<UndirectedEdge> &edges) const;

	template<typename T>
	void composeNodeValues(std::ostream &out, const T &graph) const;
	template<typename T>
	void composeEdgeValues(std::ostream &out, const T &graph) const;

};

} // namespace graph

#endif // GRAPH_TO_STRING_COMPOSER_H_
