#ifndef GRAPH_TO_XML_COMPOSER_H_
#define GRAPH_TO_XML_COMPOSER_H_

#include <set>
#include <list>

#include "Graph.h"
#include "common/GraphElement.h"
#include "directed/DirectedGraph.h"
#include "undirected/UndirectedGraph.h"
#include "../std/visitor.hpp"
#include "../sax/Token.h"

namespace alib {

template<typename T>
struct xmlApi;

} // namespace alib

namespace graph {

// This class contains methods to print XML representation of graph to the output stream.
class GraphToXMLComposer : public GraphElement::const_visitor_type
{
private:
	void Visit(void *data, const Node &node) const;
	void Visit(void *data, const DirectedEdge &edge) const;
	void Visit(void *data, const UndirectedEdge &edge) const;

	void compose(std::list<sax::Token> &out, const GraphBase &graph) const;
	void compose(std::list<sax::Token> &out, const Graph &graph) const;
	void compose(std::list<sax::Token> &out, const DirectedGraph &graph) const;
	void compose(std::list<sax::Token> &out, const UndirectedGraph &graph) const;

	void composeRepresentation(std::list<sax::Token> &out, const std::string &representation) const;
	void composeNodes(std::list<sax::Token> &out, const std::set<Node> &nodes) const;

	template<typename T>
	void composeEdges(std::list<sax::Token> &out, const std::set<T> &edges) const;

	template<typename T>
	void composeNodeValues(std::list<sax::Token> &out, const T &graph) const;
	template<typename T>
	void composeEdgeValues(std::list<sax::Token> &out, const T &graph) const;

	template<typename T> friend class alib::xmlApi;
};

} // namespace graph

#endif // GRAPH_TO_XML_COMPOSER_H_
