#include "Node.h"
#include "../../label/Label.h"

#include <sstream>

namespace graph {

Node::Node()
	: name(label::labelFrom(std::string()))
{
}

Node::Node(const label::Label &name)
	: name(name)
{
}

Node::Node(label::Label &&name)
	: name(std::move(name))
{
}

Node::Node(int number)
	: name(label::labelFrom(number))
{
}

Node::Node(char character)
	: name(label::labelFrom(character))
{
}

Node::Node(const std::string &name)
	: name(label::labelFrom(name))
{
}

Node::Node(const Node &other)
	: name(other.name)
{
}

Node::Node(Node &&other) noexcept
	: name(std::move(other.name))
{
}

Node &Node::operator=(const Node &other)
{
	if (this == &other) {
		return *this;
	}

	*this = Node(other);
	return *this;
}

Node &Node::operator=(Node &&other)
{
	std::swap(name, other.name);
	return *this;
}

GraphElement *Node::clone() const
{
	return new Node(*this);
}

GraphElement *Node::plunder() &&
{
	return new Node(std::move(*this));
}

const label::Label &Node::getName() const
{
	return name;
}

int Node::compare(const Node &other) const
{
	return name.getData().compare(other.name.getData());
}

void Node::operator>>(std::ostream &out) const
{
	dump(out);
}

Node::operator std::string() const
{
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

std::ostream &operator<<(std::ostream &out, const Node &node)
{
	node.dump(out);
	return out;
}

void Node::dump(std::ostream &os) const
{
	os << "(Node " << name << ")";
}

} // namespace graph

