#ifndef NODE_H_
#define NODE_H_

#include "Macros.h"
#include "GraphElement.h"
#include "../../label/Label.h"

namespace graph {

class Node : public std::acceptor<Node, GraphElement, GraphElement>
{
public:
	explicit Node();
	explicit Node(const label::Label &name);
	explicit Node(label::Label &&name);
	explicit Node(int number);
	explicit Node(char character);
	explicit Node(const std::string &name);

	Node(const Node &other);
	Node(Node &&other) noexcept;
	Node &operator=(const Node &other);
	Node &operator=(Node &&other);

	GraphElement *clone() const;
	GraphElement *plunder() &&;

	const label::Label &getName() const;

	int compare(const GraphElement &other) const override
	{
		return -other.compare(*this);
	}

	int compare(const Node &other) const;

	void operator>>(std::ostream &out) const override;

	explicit operator std::string() const override;

	int selfTypeId() const override
	{
		return typeId<Node>();
	}

	friend std::ostream &operator<<(std::ostream &out, const Node &node);

private:
	void dump(std::ostream &os) const;

	label::Label name;
};

} // namespace graph

GRAPH_DEFINE_STD_HASH(Node)
GRAPH_DEFINE_STD_COMPARE(Node)

#endif // NODE_H_
