#ifndef ADJACENCY_MATRIX_DIRECTED_GRAPH_H_
#define ADJACENCY_MATRIX_DIRECTED_GRAPH_H_

#include <set>
#include <vector>
#include <unordered_map>

#include "IDirectedGraph.h"

namespace graph {

/*
 *  AdjacencyMatrix representation
 *
 *  unordered_map:
 *    [ -- ][ n1 ][ n2 ][ n3 ][ n4 ]
 *    [ n1 ]  0     2     0     1
 *    [ n2 ]  1     0     0     1
 *    [ n3 ]  0     0     0     1
 *    [ n4 ]  1     0     1     0
 *
 *  edges:
 *    n1 -> n2, n1 -> n2, n1 -> n4
 *    n2 -> n1, n2 -> n4
 *    n3 -> n4
 *    n4 -> n1, n4 -> n3
 *
 */

class AdjacencyMatrixDirectedGraph : public IDirectedGraph
{
public:
	std::set<Node> allNodes() const override;
	std::set<DirectedEdge> allEdges() const override;

	bool hasNode(const Node &node) const override;
	bool hasEdge(const DirectedEdge &edge) const override;

	bool hasEdge(const Node &from, const Node &to) const override;
	std::set<DirectedEdge> findEdges(const Node &from, const Node &to) const override;

	std::set<Node> neighbors(const Node &node) const override;
	std::set<DirectedEdge> neighborEdges(const Node &node) const override;

	bool addNode(const Node &node) override;
	bool removeNode(const Node &node) override;

	bool addEdge(const DirectedEdge &edge) override;
	bool removeEdge(const DirectedEdge &edge) override;

	void copy(IDirectedGraph *other) override;
	int compare(IDirectedGraph *other) const override;

private:
	std::set<DirectedEdge> edges;
	std::unordered_map<Node, std::unordered_map<Node, int>> adj;
};

} // namespace graph

#endif // ADJACENCY_MATRIX_DIRECTED_GRAPH_H_
