#include "DirectedEdge.h"

#include <sstream>

namespace graph {

DirectedEdge::DirectedEdge()
	: name(label::labelFrom('d'))
{
}

DirectedEdge::DirectedEdge(const Node &from, const Node &to)
	: from(from)
	, to(to)
	, name(label::labelFrom('d'))
{
}

DirectedEdge::DirectedEdge(Node &&from, Node &&to)
	: from(std::move(from))
	, to(std::move(to))
	, name(label::labelFrom('d'))
{
}

DirectedEdge::DirectedEdge(const Node &from, const Node &to, const label::Label &name)
	: from(from)
	, to(to)
	, name(name)
{
}

DirectedEdge::DirectedEdge(Node &&from, Node &&to, label::Label &&name)
	: from(std::move(from))
	, to(std::move(to))
	, name(std::move(name))
{
}

DirectedEdge::DirectedEdge(const Node &from, const Node &to, int number)
	: from(from)
	, to(to)
	, name(label::labelFrom(number))
{
}

DirectedEdge::DirectedEdge(Node &&from, Node &&to, int number)
	: from(std::move(from))
	, to(std::move(to))
	, name(label::labelFrom(number))
{
}

DirectedEdge::DirectedEdge(const Node &from, const Node &to, char character)
	: from(from)
	, to(to)
	, name(label::labelFrom(character))
{
}

DirectedEdge::DirectedEdge(Node &&from, Node &&to, char character)
	: from(std::move(from))
	, to(std::move(to))
	, name(label::labelFrom(character))
{
}

DirectedEdge::DirectedEdge(const Node &from, const Node &to, const std::string &name)
	: from(from)
	, to(to)
	, name(label::labelFrom(name))
{
}

DirectedEdge::DirectedEdge(Node &&from, Node &&to, const std::string &name)
	: from(std::move(from))
	, to(std::move(to))
	, name(label::labelFrom(name))
{
}

DirectedEdge::DirectedEdge(const DirectedEdge &other)
	: from(other.from)
	, to(other.to)
	, name(other.name)
{
}

DirectedEdge::DirectedEdge(DirectedEdge &&other) noexcept
	: from(std::move(other.from))
	, to(std::move(other.to))
	, name(std::move(other.name))
{
}

DirectedEdge &DirectedEdge::operator=(const DirectedEdge &other)
{
	if (this == &other) {
		return *this;
	}

	*this = DirectedEdge(other);
	return *this;
}

DirectedEdge &DirectedEdge::operator=(DirectedEdge &&other)
{
	std::swap(from, other.from);
	std::swap(to, other.to);
	std::swap(name, other.name);
	return *this;
}

GraphElement *DirectedEdge::clone() const
{
	return new DirectedEdge(*this);
}

GraphElement *DirectedEdge::plunder() &&
{
	return new DirectedEdge(std::move(*this));
}

const Node &DirectedEdge::getFromNode() const
{
	return from;
}

const Node &DirectedEdge::getToNode() const
{
	return to;
}

const label::Label &DirectedEdge::getName() const
{
	return name;
}

int DirectedEdge::compare(const DirectedEdge &other) const
{
	int res = name.getData().compare(other.name.getData());
	if (res == 0) {
		res = from.compare(other.from);
		if (res == 0) {
			res = to.compare(other.to);
		}
	}
	return res;
}

void DirectedEdge::operator>>(std::ostream &out) const
{
	dump(out);
}

DirectedEdge::operator std::string() const
{
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

std::ostream &operator<<(std::ostream &out, const DirectedEdge &node)
{
	node.dump(out);
	return out;
}

void DirectedEdge::dump(std::ostream &os) const
{
	os << "(DirectedEdge " << from << " -> " << to << ")";
}

} // namespace graph

