#ifndef DIRECTED_EDGE_H_
#define DIRECTED_EDGE_H_

#include "../common/GraphElement.h"
#include "../common/Macros.h"
#include "../common/Node.h"

namespace graph {

class DirectedEdge : public std::acceptor<DirectedEdge, GraphElement, GraphElement>
{
public:
	explicit DirectedEdge();
	explicit DirectedEdge(const Node &from, const Node &to);
	explicit DirectedEdge(Node &&from, Node &&to);
	explicit DirectedEdge(const Node &from, const Node &to, const label::Label &name);
	explicit DirectedEdge(Node &&from, Node &&to, label::Label &&name);
	explicit DirectedEdge(const Node &from, const Node &to, int number);
	explicit DirectedEdge(Node &&from, Node &&to, int number);
	explicit DirectedEdge(const Node &from, const Node &to, char character);
	explicit DirectedEdge(Node &&from, Node &&to, char character);
	explicit DirectedEdge(const Node &from, const Node &to, const std::string &name);
	explicit DirectedEdge(Node &&from, Node &&to, const std::string &name);

	DirectedEdge(const DirectedEdge &other);
	DirectedEdge(DirectedEdge &&other) noexcept;
	DirectedEdge &operator=(const DirectedEdge &other);
	DirectedEdge &operator=(DirectedEdge &&other);

	GraphElement *clone() const;
	GraphElement *plunder() &&;

	const Node &getFromNode() const;
	const Node &getToNode() const;
	const label::Label &getName() const;

	int compare(const GraphElement &other) const override
	{
		return -other.compare(*this);
	}

	int compare(const DirectedEdge &other) const;

	void operator>>(std::ostream &out) const override;

	explicit operator std::string() const override;

	int selfTypeId() const override
	{
		return typeId<DirectedEdge>();
	}

	friend std::ostream &operator<<(std::ostream &out, const DirectedEdge &node);

private:
	void dump(std::ostream &os) const;

	Node from;
	Node to;
	label::Label name;
};

} // namespace graph

GRAPH_DEFINE_STD_HASH(DirectedEdge)
GRAPH_DEFINE_STD_COMPARE(DirectedEdge)

#endif // DIRECTED_EDGE_H_
