#ifndef DIRECTED_GRAPH_H_
#define DIRECTED_GRAPH_H_

#include <unordered_map>

#include "../GraphBase.h"
#include "../GraphRepresentation.h"
#include "../common/Macros.h"
#include "../common/Node.h"
#include "DirectedEdge.h"

namespace graph {

class IDirectedGraph;

class DirectedGraph : public std::acceptor<DirectedGraph, VisitableGraphBase, std::acceptor<DirectedGraph, alib::VisitableObjectBase, GraphBase>>
{
public:
	explicit DirectedGraph(REPRESENTATION representation = REPRESENTATION::ADJACENCY_LIST);
	~DirectedGraph() noexcept;

	DirectedGraph(const DirectedGraph &other);
	DirectedGraph(DirectedGraph &&other) noexcept;
	DirectedGraph &operator=(const DirectedGraph &other);
	DirectedGraph &operator=(DirectedGraph &&other) noexcept;

	GraphBase *clone() const override;
	GraphBase *plunder() && override;

	REPRESENTATION getRepresentation() const;

	std::set<Node> getNodes() const;
	std::set<DirectedEdge> getEdges() const;

	bool hasNode(const Node &node) const;
	bool hasEdge(const DirectedEdge &edge) const;

	bool hasEdge(const Node &from, const Node &to) const;
	std::set<DirectedEdge> findEdges(const Node &from, const Node &to) const;

	std::set<Node> neighbors(const Node &node) const;
	std::set<DirectedEdge> neighborEdges(const Node &node) const;

	bool addNode(const Node &node);
	bool addNode(const Node &node, int value);
	bool removeNode(const Node &node);

	bool addEdge(const DirectedEdge &edge);
	bool addEdge(const DirectedEdge &edge, int value);
	bool removeEdge(const DirectedEdge &edge);

	int getNodeValue(const Node &node) const;
	void setNodeValue(const Node &node, int value);

	int getEdgeValue(const DirectedEdge &edge) const;
	void setEdgeValue(const DirectedEdge &edge, int value);

	int compare(const ObjectBase &other) const override
	{
		return -other.compare(*this);
	}

	int compare(const DirectedGraph &other) const;

	void operator>>(std::ostream &out) const override;

	explicit operator std::string() const override;

	int selfTypeId() const override
	{
		return typeId<DirectedGraph>();
	}

private:
	void init();

	REPRESENTATION representation;
	IDirectedGraph *impl;

	std::unordered_map<Node, int> nodeValues;
	std::unordered_map<DirectedEdge, int> edgeValues;

	friend class GraphToXMLComposer;
	friend class GraphToStringComposer;
};

} // namespace graph

GRAPH_DEFINE_STD_COMPARE(DirectedGraph)

#endif // DIRECTED_GRAPH_H_
