#ifndef UNDIRECTED_EDGE_H_
#define UNDIRECTED_EDGE_H_

#include "../common/GraphElement.h"
#include "../common/Macros.h"
#include "../common/Node.h"

namespace graph {

class UndirectedEdge : public std::acceptor<UndirectedEdge, GraphElement, GraphElement>
{
public:
	explicit UndirectedEdge();
	explicit UndirectedEdge(const Node &first, const Node &second);
	explicit UndirectedEdge(Node &&first, Node &&second);
	explicit UndirectedEdge(const Node &first, const Node &second, const label::Label &name);
	explicit UndirectedEdge(Node &&first, Node &&second, label::Label &&name);
	explicit UndirectedEdge(const Node &first, const Node &second, int number);
	explicit UndirectedEdge(Node &&first, Node &&second, int number);
	explicit UndirectedEdge(const Node &first, const Node &second, char character);
	explicit UndirectedEdge(Node &&first, Node &&second, char character);
	explicit UndirectedEdge(const Node &first, const Node &second, const std::string &name);
	explicit UndirectedEdge(Node &&first, Node &&second, const std::string &name);

	UndirectedEdge(const UndirectedEdge &other);
	UndirectedEdge(UndirectedEdge &&other) noexcept;
	UndirectedEdge &operator=(const UndirectedEdge &other);
	UndirectedEdge &operator=(UndirectedEdge &&other);

	GraphElement *clone() const;
	GraphElement *plunder() &&;

	const Node &getFirstNode() const;
	const Node &getSecondNode() const;
	const label::Label &getName() const;

	int compare(const GraphElement &other) const override
	{
		return -other.compare(*this);
	}

	int compare(const UndirectedEdge &other) const;

	void operator>>(std::ostream &out) const override;

	explicit operator std::string() const override;

	int selfTypeId() const override
	{
		return typeId<UndirectedEdge>();
	}

	friend std::ostream &operator<<(std::ostream &out, const UndirectedEdge &node);

private:
	void dump(std::ostream &os) const;
	std::pair<Node, Node> sortedNodes() const;

	Node first;
	Node second;
	label::Label name;
};

} // namespace graph

GRAPH_DEFINE_STD_HASH(UndirectedEdge)
GRAPH_DEFINE_STD_COMPARE(UndirectedEdge)

#endif // UNDIRECTED_EDGE_H_
