#ifndef UNDIRECTED_GRAPH_H_
#define UNDIRECTED_GRAPH_H_

#include <unordered_map>

#include "../GraphBase.h"
#include "../GraphRepresentation.h"
#include "../common/Macros.h"
#include "../common/Node.h"
#include "UndirectedEdge.h"

namespace graph {

class IUndirectedGraph;

class UndirectedGraph : public std::acceptor<UndirectedGraph, VisitableGraphBase, std::acceptor<UndirectedGraph, alib::VisitableObjectBase, GraphBase>>
{
public:
	explicit UndirectedGraph(REPRESENTATION representation = REPRESENTATION::ADJACENCY_LIST);
	~UndirectedGraph() noexcept;

	UndirectedGraph(const UndirectedGraph &other);
	UndirectedGraph(UndirectedGraph &&other) noexcept;
	UndirectedGraph &operator=(const UndirectedGraph &other);
	UndirectedGraph &operator=(UndirectedGraph &&other) noexcept;

	GraphBase *clone() const override;
	GraphBase *plunder() && override;

	REPRESENTATION getRepresentation() const;

	std::set<Node> getNodes() const;
	std::set<UndirectedEdge> getEdges() const;

	bool hasNode(const Node &node) const;
	bool hasEdge(const UndirectedEdge &edge) const;

	bool hasEdge(const Node &from, const Node &to) const;
	std::set<UndirectedEdge> findEdges(const Node &from, const Node &to) const;

	std::set<Node> neighbors(const Node &node) const;
	std::set<UndirectedEdge> neighborEdges(const Node &node) const;

	bool addNode(const Node &node);
	bool addNode(const Node &node, int value);
	bool removeNode(const Node &node);

	bool addEdge(const UndirectedEdge &edge);
	bool addEdge(const UndirectedEdge &edge, int value);
	bool removeEdge(const UndirectedEdge &edge);

	int getNodeValue(const Node &node) const;
	void setNodeValue(const Node &node, int value);

	int getEdgeValue(const UndirectedEdge &edge) const;
	void setEdgeValue(const UndirectedEdge &edge, int value);

	int compare(const ObjectBase &other) const override
	{
		return -other.compare(*this);
	}

	int compare(const UndirectedGraph &other) const;

	void operator>>(std::ostream &out) const override;

	explicit operator std::string() const override;

	int selfTypeId() const override
	{
		return typeId<UndirectedGraph>();
	}

private:
	void init();

	REPRESENTATION representation;
	IUndirectedGraph *impl;

	std::unordered_map<Node, int> nodeValues;
	std::unordered_map<UndirectedEdge, int> edgeValues;

	friend class GraphToXMLComposer;
	friend class GraphToStringComposer;
};

} // namespace graph

GRAPH_DEFINE_STD_COMPARE(UndirectedGraph)

#endif // UNDIRECTED_GRAPH_H_
