/*
 * LabelFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_FROM_XML_PARSER_H_
#define LABEL_FROM_XML_PARSER_H_

#include "../sax/FromXMLParserHelper.h"
#include <vector>
#include <set>
#include "Label.h"
#include "LabelFeatures.h"
#include "PrimitiveLabel.h"
#include "HexavigesimalLabel.h"
#include "ObjectLabel.h"
#include "LabelSetLabel.h"
#include "LabelPairLabel.h"
#include "UniqueLabel.h"
#include "../sax/Token.h"

namespace alib {

template<typename T>
struct xmlApi;

} /* namespace alib */

namespace label {

/**
 * Parser used to transform sequence of xml tokens to internal representation of Label.
 */
class LabelFromXMLParser : public sax::FromXMLParserHelper {
	Label parseLabel(std::list<sax::Token>& input, const std::set<FEATURES>&) const;
	Label parseLabel(std::list<sax::Token>& input) const;

	PrimitiveLabel parsePrimitiveLabel(std::list<sax::Token>& input) const;
	HexavigesimalLabel parseHexavigesimalLabel(std::list<sax::Token>& input) const;
	ObjectLabel parseObjectLabel(std::list<sax::Token>& input) const;
	LabelSetLabel parseLabelSetLabel(std::list<sax::Token>& input) const;
	LabelPairLabel parseLabelPairLabel(std::list<sax::Token>& input) const;
	UniqueLabel parseUniqueLabel(std::list<sax::Token>& input) const;

	template<typename T> friend class alib::xmlApi;
public:
	bool first(const std::list<sax::Token>& input) const;
};

} /* namespace label */

#endif /* LABEL_FROM_XML_PARSER_H_ */
