/*
 * LabelSetLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "LabelSetLabel.h"
#include "../std/set.hpp"
#include <sstream>

namespace label {

LabelSetLabel::LabelSetLabel(const std::set<Label>& label) : label(label) {

}

LabelSetLabel::LabelSetLabel(std::set<Label>&& label) : label(std::move(label)) {

}

LabelBase* LabelSetLabel::clone() const {
	return new LabelSetLabel(*this);
}

LabelBase* LabelSetLabel::plunder() && {
	return new LabelSetLabel(std::move(*this));
}

const std::set<Label>& LabelSetLabel::getData() const {
	return label;
}

int LabelSetLabel::compare(const LabelSetLabel& other) const {
	std::compare<std::set<Label>> comp;
	return comp(label, other.label);
}

void LabelSetLabel::operator>>(std::ostream& out) const {
	out << "(LabelSetLabel " << label << ")";
}

LabelSetLabel::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	bool first = true;
	for(const Label& subLabel : label) {
		if(!first) 
			ss << ", ";
		else
			first = false;
		ss << (std::string) subLabel;
	}
	ss << "}";
	return std::move(ss).str();
}

} /* namespace label */

