/*
 * LabelToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelToStringComposer.h"
#include "PrimitiveLabel.h"
#include "HexavigesimalLabel.h"
#include "ObjectLabel.h"
#include "LabelSetLabel.h"
#include "LabelPairLabel.h"
#include "UniqueLabel.h"

#include "../StringApi.hpp"

namespace label {

void LabelToStringComposer::Visit(void* userData, const PrimitiveLabel& label) const {
	std::ostream &out = *((std::ostream*) userData);

	alib::stringApi<primitive::Primitive>::compose(out, label.getData());
}

void LabelToStringComposer::Visit(void* userData, const HexavigesimalLabel& label) const {
	std::ostream &out = *((std::ostream*) userData);

	out << (std::string) label;
}

void LabelToStringComposer::Visit(void* userData, const ObjectLabel& label) const {
	std::ostream &out = *((std::ostream*) userData);
	out << (std::string) label;
}

void LabelToStringComposer::Visit(void* userData, const LabelSetLabel& label) const {
	std::ostream &out = *((std::ostream*) userData);

	out << '[';
	bool first = true;
	for(const Label& innerLabel : label.getData()) {
		if(!first)
			out << ", ";
		else
			first = false;
		innerLabel.getData().Accept(userData, *this);
	}
	out << ']';
}

void LabelToStringComposer::Visit(void* userData, const LabelPairLabel& label) const {
	std::ostream &out = *((std::ostream*) userData);

	out << '<';
	label.getData().first.getData().Accept(userData, *this);
	out << ", ";
	label.getData().second.getData().Accept(userData, *this);
	out << '>';
}

void LabelToStringComposer::Visit(void* userData, const UniqueLabel& label) const {
	std::ostream &out = *((std::ostream*) userData);

	out << '<';
	label.getLabel().getData().Accept(userData, *this);
	out << ", ";
	primitive::PrimitiveToStringComposer composer;
	label.getId().Accept(userData, composer);
	out << '>';
}

void LabelToStringComposer::compose(std::ostream& output, const Label& label) const {
	label.getData().Accept((void*) &output, *this);
}

} /* namespace label */
