/*
 * NextLabel.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "NextLabel.h"
#include "../primitive/NextPrimitive.h"
#include "PrimitiveLabel.h"
#include "HexavigesimalLabel.h"
#include "../std/itos.h"
#include "../exception/AlibException.h"
#include "LabelSetLabel.h"
#include "ObjectLabel.h"
#include "LabelPairLabel.h"
#include "UniqueLabel.h"

namespace label {

void NextLabel::Visit(void* userData, const PrimitiveLabel& label) {
	Label* &out = *((Label**) userData);

	primitive::NextPrimitive next;
	out = new Label(PrimitiveLabel(next.nextPrimitive(label.getData())));
}

void NextLabel::Visit(void* userData, const HexavigesimalLabel& label) {
	Label* &out = *((Label**) userData);

	out = new Label(HexavigesimalLabel(label.getData() + 1));
}

void NextLabel::Visit(void* userData, const ObjectLabel& label) {
	Label* &out = *((Label**) userData);

	primitive::Integer newId(0);
	out = new Label(UniqueLabel(Label(label), newId));
}

void NextLabel::Visit(void* userData, const LabelSetLabel& label) {
	Label* &out = *((Label**) userData);

	primitive::Integer newId(0);
	out = new Label(UniqueLabel(Label(label), newId));
}

void NextLabel::Visit(void* userData, const LabelPairLabel& label) {
	Label* &out = *((Label**) userData);

	primitive::Integer newId(0);
	out = new Label(UniqueLabel(Label(label), newId));
}

void NextLabel::Visit(void* userData, const UniqueLabel& label) {
	Label* &out = *((Label**) userData);

	primitive::NextPrimitive next;
	primitive::Integer newId(label.getId().getData() + 1);
	out = new Label(UniqueLabel(label.getLabel(), newId));
}

Label NextLabel::nextLabel(const Label& label) {
	Label* out;
	label.getData().Accept((void*) &out, *this);

	Label nextLabel(std::move(*out));
	delete out;
	return std::move(nextLabel);
}

} /* namespace label */
