/*
 * UniqueLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "UniqueLabel.h"
#include "../std/set.hpp"
#include <sstream>

namespace label {

UniqueLabel::UniqueLabel(const Label& label, primitive::Integer& id) : label(label), id(id) {

}

LabelBase* UniqueLabel::clone() const {
	return new UniqueLabel(*this);
}

LabelBase* UniqueLabel::plunder() && {
	return new UniqueLabel(std::move(*this));
}

const Label& UniqueLabel::getLabel() const {
	return label;
}

const primitive::Integer& UniqueLabel::getId() const {
	return id;
}

int UniqueLabel::compare(const UniqueLabel& other) const {
	int res = label.getData().compare(other.label.getData());
	if(res == 0) res = id.compare(other.id);
	return res;
}

void UniqueLabel::operator>>(std::ostream& out) const {
	out << "(UniqueLabel " << label << ", " << id << ")";
}

UniqueLabel::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	ss << label;
	ss << ", ";
	ss << id;
	ss << "}";
	return std::move(ss).str();
}

} /* namespace label */

