/*
 * PrimitiveToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "PrimitiveToStringComposer.h"
#include "Integer.h"
#include "String.h"
#include "Character.h"
#include "Unsigned.h"
#include "Bool.h"

namespace primitive {

void PrimitiveToStringComposer::Visit(void* userData, const primitive::String& primitive) const {
	std::ostream &out = *((std::ostream*) userData);

	auto replace = [](std::string& str, const std::string& what, const std::string& with) {
		size_t index = 0;
		while((index = str.find(what, index)) != std::string::npos) {
			str.replace(index, what.length(), with);
			index += with.length();
		}
	};
	
	std::string tmp = primitive.getData();
	replace(tmp, "'", "\\'" );
	out << '\'' << tmp << '\'';
}

void PrimitiveToStringComposer::Visit(void* userData, const Character& primitive) const {
	std::ostream &out = *((std::ostream*) userData);

	out << primitive.getData();
}

void PrimitiveToStringComposer::Visit(void* userData, const Integer& primitive) const {
	std::ostream &out = *((std::ostream*) userData);

	out << primitive.getData();
}

void PrimitiveToStringComposer::Visit(void* userData, const Unsigned& primitive) const {
	std::ostream &out = *((std::ostream*) userData);

	out << primitive.getData();
}

void PrimitiveToStringComposer::Visit(void* userData, const Bool& primitive) const {
	std::ostream &out = *((std::ostream*) userData);

	out << (primitive.getData() ? "true" : "false");
}

void PrimitiveToStringComposer::compose(std::ostream& out, const Primitive& primitive) const {
	primitive.getData().Accept((void*) &out, *this);
}

} /* namespace primitive */
