/*
 * RegExp.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "RegExp.h"
#include "../alphabet/Symbol.h"
#include "unbounded/UnboundedRegExpSymbol.h"
#include "unbounded/UnboundedRegExpEmpty.h"
#include "unbounded/UnboundedRegExpConcatenation.h"

namespace regexp {

regexp::RegExp regexpFrom( const std::string& string ) {
	regexp::UnboundedRegExpConcatenation con;
	for(const auto& symbol : string) {
		con.appendElement( regexp::UnboundedRegExpSymbol( alphabet::symbolFrom(symbol)));
	}
	return regexp::RegExp { regexp::UnboundedRegExp ( con ) };
}

regexp::RegExp regexpFrom( const char* string ) {
	return regexpFrom( (std::string) string );
}

regexp::RegExp regexpFrom( const string::LinearString& string ) {
	regexp::UnboundedRegExpConcatenation con;
	for(const auto& symbol : string.getContent()) {
		con.appendElement( regexp::UnboundedRegExpSymbol(symbol));
	}
	return regexp::RegExp { regexp::UnboundedRegExp ( con ) };
}

regexp::RegExp regexpFrom( const alphabet::Symbol& symbol ) {
	return regexp::RegExp { regexp::UnboundedRegExp( regexp::UnboundedRegExpSymbol(symbol) ) };
}

regexp::RegExp regexpFrom( ) {
	return regexp::RegExp { regexp::UnboundedRegExp( regexp::UnboundedRegExpEmpty { } ) };
}

} /* namespace regexp */

