/*
 * RegExpToStringComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef REX_EXP_TO_STRING_COMPOSER_H_
#define REX_EXP_TO_STRING_COMPOSER_H_

#include <string>
#include "RegExp.h"
#include "unbounded/UnboundedRegExpElements.h"
#include "formal/FormalRegExpElements.h"

namespace regexp {

class RegExpToStringComposer : public VisitableRegExpBase::const_visitor_type, UnboundedRegExpElement::const_visitor_type, FormalRegExpElement::const_visitor_type {
	void Visit(void*, const UnboundedRegExpAlternation& alternation) const;
	void Visit(void*, const UnboundedRegExpConcatenation& concatenation) const;
	void Visit(void*, const UnboundedRegExpIteration& iteration) const;
	void Visit(void*, const UnboundedRegExpSymbol& symbol) const;
	void Visit(void*, const UnboundedRegExpEpsilon& epsilon) const;
	void Visit(void*, const UnboundedRegExpEmpty& empty) const;


	void Visit(void*, const FormalRegExpAlternation& alternation) const;
	void Visit(void*, const FormalRegExpConcatenation& concatenation) const;
	void Visit(void*, const FormalRegExpIteration& iteration) const;
	void Visit(void*, const FormalRegExpSymbol& symbol) const;
	void Visit(void*, const FormalRegExpEpsilon& epsilon) const;
	void Visit(void*, const FormalRegExpEmpty& empty) const;


	void Visit(void*, const UnboundedRegExp& empty) const;
	void Visit(void*, const FormalRegExp& empty) const;


	enum class Priority {
		ALTERNATION,
		CONCATENATION,
		FACTOR
	};
public:
	/**
	 * Composes string representation of RegExp.
	 * @param regexp RegExp to print
	 * @returns string representation of regexp
	 */
	void compose(std::ostream& out, const RegExp& regexp) const;
};

} /* namespace regexp */

#endif /* REX_EXP_TO_STRING_COMPOSER_H_ */
