/*
 * RegExpAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "RegExpAlphabet.h"

#include <algorithm>

namespace regexp {

const std::set<alphabet::Symbol>& RegExpAlphabet::getAlphabet() const {
	return alphabet;
}

bool RegExpAlphabet::addSymbolToAlphabet(const alphabet::Symbol& symbol) {
	return alphabet.insert(symbol).second;
}

void RegExpAlphabet::addSymbolsToAlphabet(const std::set<alphabet::Symbol>& symbols) {
	for(const alphabet::Symbol& addedSymbol : symbols) {
		addSymbolToAlphabet(addedSymbol);
	}
}

void RegExpAlphabet::setAlphabet(const std::set<alphabet::Symbol>& newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(alphabet.begin(), alphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(newSymbols.begin(), newSymbols.end(), alphabet.begin(), alphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeSymbolFromAlphabet(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addSymbolToAlphabet(addedSymbol);
	}
}

} /* namespace regexp */

