/*
 * FormalRegExpAlternation.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef FORMAL_REG_EXP_ALTERNATION_H_
#define FORMAL_REG_EXP_ALTERNATION_H_

#include <vector>
#include "FormalRegExpElement.h"

namespace regexp {

namespace simplify {

class RegExpOptimize;

} /* namespace simplify */

/**
 * Represents alternation operator in the regular expression. Contains list of FormalRegExpElement
 * as operands of the operator.
 */
class FormalRegExpAlternation: public std::acceptor<FormalRegExpAlternation, FormalRegExpElement, FormalRegExpElement> {
protected:
	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* cloneAsUnbounded() const;
	
	FormalRegExpElement* left;
	FormalRegExpElement* right;

	/**
	 * @copydoc FormalRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	
	/**
	 * @copydoc FormalRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const FormalRegExp * regexp );
	
	/**
	 * @copydoc FormalRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;
	
public:
	explicit FormalRegExpAlternation(FormalRegExpElement&& left, FormalRegExpElement&& right);
	explicit FormalRegExpAlternation(const FormalRegExpElement& left, const FormalRegExpElement& right);
	
	FormalRegExpAlternation(const FormalRegExpAlternation& other);
	FormalRegExpAlternation(FormalRegExpAlternation&& other) noexcept;
	FormalRegExpAlternation& operator =(const FormalRegExpAlternation& other);
	FormalRegExpAlternation& operator =(FormalRegExpAlternation&& other);

	virtual ~FormalRegExpAlternation() noexcept;
	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual FormalRegExpElement* clone() const;
	
	/**
	 * @copydoc FormalRegExpElement::plunder() const
	 */
	virtual FormalRegExpElement* plunder() &&;

	/**
	 * @return elements
	 */
	const FormalRegExpElement & getLeftElement() const;
	const FormalRegExpElement & getRightElement() const;
	
	/**
	 * @return elements
	 */
	FormalRegExpElement & getLeftElement();
	FormalRegExpElement & getRightElement();

	/**
	 * @param element to append
	 */
	void setLeftElement(const FormalRegExpElement& element);
	void setLeftElement(FormalRegExpElement&& element);

	/**
	 * @param element to append
	 */
	void setRightElement(const FormalRegExpElement& element);
	void setRightElement(FormalRegExpElement&& element);

	virtual int compare(const FormalRegExpElement& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const FormalRegExpAlternation&) const;
	
	/**
	 * @copydoc FormalRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	friend class simplify::RegExpOptimize;

	virtual explicit operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<FormalRegExpAlternation>();
	}
};

} /* namespace regexp */

#endif /* FORMAL_REG_EXP_ALTERNATION_H_ */
