/*
 * FormalRegExpEmpty.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef FORMAL_REG_EXP_EMPTY_H_
#define FORMAL_REG_EXP_EMPTY_H_

#include "FormalRegExpElement.h"
#include <sstream>

namespace regexp {

/**
 * Represents empty regular expression in the regular expression.
 */
class FormalRegExpEmpty: public std::acceptor<FormalRegExpEmpty, FormalRegExpElement, FormalRegExpElement> {
protected:
	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* cloneAsUnbounded() const;
	
	/**
	 * @copydoc FormalRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	
	/**
	 * @copydoc FormalRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const FormalRegExp * regexp );
	
	/**
	 * @copydoc RegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:
	explicit FormalRegExpEmpty();
	FormalRegExpEmpty(const FormalRegExpEmpty& other);
	FormalRegExpEmpty(FormalRegExpEmpty&& other) noexcept;
	FormalRegExpEmpty& operator =(const FormalRegExpEmpty& other);
	FormalRegExpEmpty& operator =(FormalRegExpEmpty&& other) noexcept;

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual FormalRegExpElement* clone() const;
	
	/**
	 * @copydoc FormalRegExpElement::plunder() const
	 */
	virtual FormalRegExpElement* plunder() &&;

	virtual int compare(const FormalRegExpElement& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const FormalRegExpEmpty&) const;
	
	/**
	 * @copydoc FormalRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	virtual explicit operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<FormalRegExpEmpty>();
	}
};

} /* namespace regexp */

#endif /* FORMAL_REG_EXP_EMPTY_H_ */
