/*
 * FormalRegExpSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef FORMAL_REG_EXP_SYMBOL_H_
#define FORMAL_REG_EXP_SYMBOL_H_

#include "../../label/Label.h"
#include "FormalRegExpElement.h"
#include "../../alphabet/LabeledSymbol.h"

namespace regexp {

/**
 * Represents symbol in the regular expression. Contains name of the symbol.
 */
class FormalRegExpSymbol : public std::acceptor<FormalRegExpSymbol, FormalRegExpElement, FormalRegExpElement> {
protected:
	alphabet::Symbol symbol;

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* cloneAsUnbounded() const;
	
	/**
	 * @copydoc FormalRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
		
	/**
	 * @copydoc FormalRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const FormalRegExp * regexp );
	
	/**
	 * @copydoc FormalRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;
public:
	explicit FormalRegExpSymbol(int number);
	explicit FormalRegExpSymbol(char character);
	explicit FormalRegExpSymbol(const std::string& label);

	explicit FormalRegExpSymbol(const alphabet::Symbol& symbol);
	explicit FormalRegExpSymbol(alphabet::Symbol&& symbol);
	
	FormalRegExpSymbol(const FormalRegExpSymbol& other);
	FormalRegExpSymbol(FormalRegExpSymbol&& other) noexcept;
	FormalRegExpSymbol& operator=(const FormalRegExpSymbol& other);
	FormalRegExpSymbol& operator=(FormalRegExpSymbol&& other);

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual FormalRegExpElement* clone() const;
	
	/**
	 * @copydoc FormalRegExpElement::plunder() const
	 */
	virtual FormalRegExpElement* plunder() &&;

	
	bool operator==(const alphabet::Symbol&) const;
	friend bool operator==(const alphabet::Symbol&, const FormalRegExpSymbol&);

	virtual int compare(const FormalRegExpElement& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const FormalRegExpSymbol&) const;
	
	/**
	 * @copydoc FormalRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;
	
	/**
	 * Returns the string representation of RegExp Symbol.
	 */
	const alphabet::Symbol& getSymbol() const;

	virtual explicit operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<FormalRegExpSymbol>();
	}
};

} /* namespace regexp */

#endif /* FORMAL_REG_EXP_SYMBOL_H_ */

