/*
 * UnboundedRegExpAlternation.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef UNBOUNDED_REG_EXP_ALTERNATION_H_
#define UNBOUNDED_REG_EXP_ALTERNATION_H_

#include <vector>
#include "UnboundedRegExpElement.h"

namespace regexp {

namespace simplify {

class RegExpOptimize;

} /* namespace simplify */

/**
 * Represents alternation operator in the regular expression. Contains list of UnboundedRegExpElement
 * as operands of the operator.
 */
class UnboundedRegExpAlternation: public std::acceptor<UnboundedRegExpAlternation, UnboundedRegExpElement, UnboundedRegExpElement> {
protected:
	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	virtual FormalRegExpElement* cloneAsFormal() const;

	std::vector<UnboundedRegExpElement*> elements;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc UnboundedRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const UnboundedRegExp * regexp );

	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:
	explicit UnboundedRegExpAlternation();

	UnboundedRegExpAlternation(const UnboundedRegExpAlternation& other);
	UnboundedRegExpAlternation(UnboundedRegExpAlternation&& other) noexcept;
	UnboundedRegExpAlternation& operator =(const UnboundedRegExpAlternation& other);
	UnboundedRegExpAlternation& operator =(UnboundedRegExpAlternation&& other);
	virtual ~UnboundedRegExpAlternation() noexcept;

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* clone() const;

	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	virtual UnboundedRegExpElement* plunder() &&;

	/**
	 * @return elements
	 */
	const std::vector<const UnboundedRegExpElement*> & getElements() const;

	/**
	 * @return elements
	 */
	const std::vector<UnboundedRegExpElement*> & getElements();

	/**
	 * @param element to append
	 */
	void appendElement(const UnboundedRegExpElement& element);

	/**
	 * @param element to append
	 */
	void appendElement(UnboundedRegExpElement&& element);

	virtual int compare(const UnboundedRegExpElement& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const UnboundedRegExpAlternation&) const;

	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	friend class simplify::RegExpOptimize;

	virtual explicit operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<UnboundedRegExpAlternation>();
	}
};

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_ALTERNATION_H_ */
