/*
 * UnboundedRegExpEmpty.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef UNBOUNDED_REG_EXP_EMPTY_H_
#define UNBOUNDED_REG_EXP_EMPTY_H_

#include "UnboundedRegExpElement.h"

namespace regexp {

/**
 * Represents empty regular expression in the regular expression.
 */
class UnboundedRegExpEmpty: public std::acceptor<UnboundedRegExpEmpty, UnboundedRegExpElement, UnboundedRegExpElement> {
protected:
	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	virtual FormalRegExpElement* cloneAsFormal() const;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	
	/**
	 * @copydoc UnboundedRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const UnboundedRegExp * regexp );
	
	/**
	 * @copydoc RegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:
	explicit UnboundedRegExpEmpty();
	UnboundedRegExpEmpty(const UnboundedRegExpEmpty& other);
	UnboundedRegExpEmpty(UnboundedRegExpEmpty&& other) noexcept;
	UnboundedRegExpEmpty& operator =(const UnboundedRegExpEmpty& other);
	UnboundedRegExpEmpty& operator =(UnboundedRegExpEmpty&& other) noexcept;

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* clone() const;
	
	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	virtual UnboundedRegExpElement* plunder() &&;

	virtual int compare(const UnboundedRegExpElement& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const UnboundedRegExpEmpty&) const;
	
	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	virtual explicit operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<UnboundedRegExpEmpty>();
	}
};

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_EMPTY_H_ */
