/*
 * SaxComposeInterface.cpp
 *
 *	Created on: 8.8.2012
 *			Author: Martin Zak
 */

#include "SaxComposeInterface.h"

#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <deque>
#include "ComposerException.h"

namespace sax {

void SaxComposeInterface::printMemory(std::string& xmlOut, const std::list<Token>& in) {
	xmlBufferPtr buf = xmlBufferCreate();
	xmlTextWriterPtr writer = xmlNewTextWriterMemory(buf, 0);

	SaxComposeInterface::xmlSAXUserPrint(writer, in);

	xmlFreeTextWriter(writer);
	xmlOut = (const char*) buf->content;
	xmlBufferFree(buf);
}

void SaxComposeInterface::printFile(const std::string& filename, const std::list<Token>& in) {
	xmlTextWriterPtr writer = xmlNewTextWriterFilename(filename.c_str(), 0);

	SaxComposeInterface::xmlSAXUserPrint(writer, in);

	xmlFreeTextWriter(writer);
}

void SaxComposeInterface::printStdout(const std::list<Token>& in) {
	SaxComposeInterface::printStream(std::cout, in);
}

void SaxComposeInterface::printStream(std::ostream& out, const std::list<Token>& in) {
	xmlBufferPtr buf = xmlBufferCreate();
	xmlTextWriterPtr writer = xmlNewTextWriterMemory(buf, 0);

	SaxComposeInterface::xmlSAXUserPrint(writer, in);

	xmlFreeTextWriter(writer);
	out << (const char*) buf->content;
	xmlBufferFree(buf);
}

void SaxComposeInterface::xmlSAXUserPrint(xmlTextWriterPtr writer, const std::list<Token>& in) {
	xmlTextWriterStartDocument(writer, NULL, NULL, NULL);
	std::deque<Token> stack;
	for(const Token& token : in) {
		switch(token.getType()) {
		case Token::TokenType::START_ELEMENT:
			xmlTextWriterStartElement(writer, (const xmlChar*) token.getData().c_str());
			stack.push_back(token);
			break;
		case Token::TokenType::END_ELEMENT:
			if(stack.empty() || stack.back().getData() != token.getData() || stack.back().getType() != Token::TokenType::START_ELEMENT) {
				throw ComposerException(Token(stack.back().getData(), Token::TokenType::END_ELEMENT), token);
			}
			stack.pop_back();
			xmlTextWriterEndElement(writer);
			break;
		case Token::TokenType::CHARACTER:
			xmlTextWriterWriteString(writer, (const xmlChar*) token.getData().c_str());
			break;
		case Token::TokenType::START_ATTRIBUTE:
			xmlTextWriterStartAttribute(writer, (const xmlChar*) token.getData().c_str());
			stack.push_back(token);
			break;
		case Token::TokenType::END_ATTRIBUTE:
			if(stack.empty() || stack.back().getData() != token.getData() || stack.back().getType() != Token::TokenType::START_ATTRIBUTE) {
				throw ComposerException(Token(stack.back().getData(), Token::TokenType::END_ATTRIBUTE), token);
			}
			stack.pop_back();
			xmlTextWriterEndAttribute(writer);
			break;
		}
	}
	xmlTextWriterEndDocument(writer);
}

} /* namespace sax */

