/*
 * SaxParseInterface.cpp
 *
 *  Created on: 8.8.2012
 *      Author: Martin Zak
 */

#include "SaxParseInterface.h"

#include <string>
#include <cstring>
#include <cstdlib>
#include <iostream>
#include <algorithm>
#include "../exception/AlibException.h"

namespace sax {

void SaxParseInterface::initSAXHandler(xmlSAXHandler& handler) {
	memset(&handler, 0, sizeof(handler));
	handler.initialized = XML_SAX2_MAGIC;

	handler.startElement = &sax::SaxParseInterface::startElement;
	handler.endElement = &sax::SaxParseInterface::endElement;
	handler.characters = &sax::SaxParseInterface::characters;
}

void SaxParseInterface::parseMemory(const std::string& xmlIn, std::list<Token>& out) {
	xmlSAXHandler handler;
	initSAXHandler(handler);

	int result = xmlSAXUserParseMemory(&handler, (void*) &out, xmlIn.c_str(), xmlIn.length());
	xmlCleanupParser();

	if (result != 0) {
		throw exception::AlibException("Cannot parse the XML string." + xmlIn);
	}
}

void SaxParseInterface::parseFile(const std::string& filename, std::list<Token>& out) {
	xmlSAXHandler handler;
	initSAXHandler(handler);

	int result = xmlSAXUserParseFile(&handler, (void*) &out, filename.c_str());
	xmlCleanupParser();

	if (result != 0) {
		throw exception::AlibException("Cannot parse the XML file " + filename);
	}
}

void SaxParseInterface::parseStdin(std::list<Token>& out) {
	SaxParseInterface::parseStream(std::cin, out);
}

void SaxParseInterface::parseStream(std::istream& in, std::list<Token>& out) {
	std::string input(std::istreambuf_iterator<char>(in), (std::istreambuf_iterator<char>()));
	SaxParseInterface::parseMemory(input, out);
}

void SaxParseInterface::characters(void * userData, const xmlChar * ch, int len) {
	std::list<Token> &out = *((std::list<Token>*) userData);
	std::string tmp((const char*) ch, len);

	if(! std::all_of(tmp.begin(), tmp.end(), isspace)) out.push_back(Token(std::move(tmp), Token::TokenType::CHARACTER));
}

void SaxParseInterface::startElement(void* userData, const xmlChar* name, const xmlChar** attrs) {
	std::list<Token> &out = *((std::list<Token>*) userData);
	out.push_back(Token((const char*) name, Token::TokenType::START_ELEMENT));

	while(attrs && *attrs && *(attrs+1)) {
		out.push_back(Token((const char*) *attrs, Token::TokenType::START_ATTRIBUTE));

		out.push_back(Token((const char*) *(attrs + 1), Token::TokenType::CHARACTER));

		out.push_back(Token((const char*) *attrs, Token::TokenType::END_ATTRIBUTE));
		attrs+=2;
	}
}

void SaxParseInterface::endElement(void * userData, const xmlChar * name) {
	std::list<Token> &out = *((std::list<Token>*) userData);
	out.push_back(Token((const  char*) name, Token::TokenType::END_ELEMENT));
}

} /* namespace sax */
