#ifndef __MAP_HPP_
#define __MAP_HPP_

#include <map>

namespace std {

template< class T, class R >
std::ostream& operator<<(std::ostream& out, const std::map<T, R>& map);

}

#include "set.hpp"
#include "pair.hpp"
#include "list.hpp"
#include "tuple.hpp"
#include "vector.hpp"
#include <iostream>

namespace std {

template< class T, class R >
std::ostream& operator<<(std::ostream& out, const std::map<T, R>& map) {
	out << "{";

	bool first = true;
	for(const std::pair<T, R>& item : map) {
		if(!first) out << ", ";
		first = false;
		out << item;
	}

	out << "}";
	return out;
}

template<typename map_type>
class key_iterator : public map_type::const_iterator {
public:
	typedef typename map_type::const_iterator map_iterator;
	typedef typename map_iterator::value_type::first_type key_type;

	key_iterator(const map_iterator& other) : map_iterator(other) {} ;

	const key_type& operator *() const {
		return map_iterator::operator*().first;
	}

	const key_type* operator ->() const {
		return &(map_iterator::operator*().first);
	}
};

// helpers to create iterators easier:
template<typename map_type>
key_iterator<map_type> key_begin(const map_type& m) {
	return key_iterator<map_type>(m.begin());
}

template<typename map_type>
key_iterator<map_type> key_end(const map_type& m) {
	return key_iterator<map_type>(m.end());
}

} /* namespace std */

#endif /* __MAP_HPP_ */

