/*
 * CyclicString.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef CYCLIC_STRING_H_
#define CYCLIC_STRING_H_

#include <iostream>
#include <set>
#include <vector>

#include "../std/visitor.hpp"
#include "../alphabet/Symbol.h"
#include "StringBase.h"
#include "common/StringAlphabet.h"
#include "Epsilon.h"

namespace string {

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of CyclicStringElement.
 */
class CyclicString : public std::acceptor<CyclicString, VisitableStringBase, std::acceptor<CyclicString, alib::VisitableObjectBase, StringBase> >, public StringAlphabet {
	std::vector<alphabet::Symbol> m_Data;
public:
	explicit CyclicString();
	explicit CyclicString(const std::set<alphabet::Symbol>& alphabet, const std::vector<alphabet::Symbol>& data);
	explicit CyclicString(std::set<alphabet::Symbol>&& alphabet, std::vector<alphabet::Symbol>&& data);
	explicit CyclicString(const std::vector<alphabet::Symbol>& data);
	explicit CyclicString(std::vector<alphabet::Symbol>&& data);
	explicit CyclicString(const std::string& str);
	explicit CyclicString(const char* str);
	explicit CyclicString(const Epsilon& epsilon);

	virtual StringBase* clone() const;
	virtual StringBase* plunder() &&;

	virtual bool removeSymbolFromAlphabet(const alphabet::Symbol & symbol);

	/**
	 * @return List of symbols forming string (const version).
	 */
	const std::vector<alphabet::Symbol>& getContent() const;

	void setContent(std::vector<alphabet::Symbol> data);

	/**
	 * @return true if string is an empty word (vector length is 0)
	 */
	bool isEmpty() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const LinearString& other) const;
	virtual int compare(const CyclicString& other) const;
	virtual int compare(const Epsilon& other) const;

	virtual void operator >>(std::ostream& out) const;

	virtual explicit operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<CyclicString>();
	}
};

} /* namespace string */

namespace std {

template<>
struct compare<::string::CyclicString> {
	int operator()(const ::string::CyclicString& first, const ::string::CyclicString& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* CYCLIC_STRING_H_ */

