/*
 * Epsilon.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef EPSILON_H_
#define EPSILON_H_

#include <iostream>
#include <set>
#include <vector>

#include "../std/visitor.hpp"
#include "../alphabet/Symbol.h"
#include "StringBase.h"
#include "common/StringAlphabet.h"

namespace string {

/**
 * Represents epsilon. Regular expression is stored
 * as a tree of EpsilonElement.
 */
class Epsilon : public std::acceptor<Epsilon, VisitableStringBase, std::acceptor<Epsilon, alib::VisitableObjectBase, StringBase> >, public StringAlphabet {
public:
	virtual StringBase* clone() const;
	virtual StringBase* plunder() &&;

	virtual bool removeSymbolFromAlphabet(const alphabet::Symbol & symbol);

	/**
	 * @return List of symbols forming string (const version).
	 */
	std::vector<alphabet::Symbol> getContent() const;

	/**
	 * @return true
	 */
	bool isEmpty() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const LinearString& other) const;
	virtual int compare(const CyclicString& other) const;
	virtual int compare(const Epsilon& other) const;

	virtual void operator >>(std::ostream& out) const;

	virtual explicit operator std::string() const;

	static Epsilon EPSILON;

	virtual int selfTypeId() const {
		return typeId<Epsilon>();
	}
};

} /* namespace string */

namespace std {

template<>
struct compare<::string::Epsilon> {
	int operator()(const ::string::Epsilon& first, const ::string::Epsilon& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* EPSILON_H_ */

