#include <list>
#include "LabelTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "label/Label.h"
#include "label/PrimitiveLabel.h"

#include "primitive/Integer.h"
#include "primitive/Character.h"
#include "primitive/String.h"

#include "factory/XmlDataFactory.hpp"
#include "factory/StringDataFactory.hpp"
#include <algorithm>

#define CPPUNIT_EXCLUSIVE_OR(x, y) CPPUNIT_ASSERT((!(x) && (y)) || ((x) && !(y)))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( LabelTest, "label" );
CPPUNIT_TEST_SUITE_REGISTRATION( LabelTest );

void LabelTest::setUp() {
}

void LabelTest::tearDown() {
}

void LabelTest::testCopyConstruct() {
	label::Label label = label::labelFrom("1");

	label::Label label2(label);

	CPPUNIT_ASSERT( label == label2 );

	label::Label label3(std::move(label));

	CPPUNIT_ASSERT( label2 == label3 );
}

void LabelTest::testEqual() {
	{
		std::string input = "1";
		label::Label label = alib::StringDataFactory::fromString<label::Label>(input);

		std::string output = alib::StringDataFactory::toString(label);

		CPPUNIT_ASSERT( input == output );

		label::Label label2 = alib::StringDataFactory::fromString<label::Label>(output);

		CPPUNIT_ASSERT( label == label2 );
	}
	{
		std::string input = "'aaaa'";
		label::Label label = alib::StringDataFactory::fromString<label::Label>(input);

		std::string output = alib::StringDataFactory::toString(label);

		CPPUNIT_ASSERT( input == output );

		label::Label label2 = alib::StringDataFactory::fromString<label::Label>(output);

		CPPUNIT_ASSERT( label == label2 );
	}
	{
		std::string input = "[1, 2, 3]";
		label::Label label = alib::StringDataFactory::fromString<label::Label>(input);

		std::string output = alib::StringDataFactory::toString(label);

		CPPUNIT_ASSERT( input == output );

		label::Label label2 = alib::StringDataFactory::fromString<label::Label>(output);

		CPPUNIT_ASSERT( label == label2 );
	}
	{
		std::string input = "<1, 2>";
		label::Label label = alib::StringDataFactory::fromString<label::Label>(input);

		std::string output = alib::StringDataFactory::toString(label);

		CPPUNIT_ASSERT( input == output );

		label::Label label2 = alib::StringDataFactory::fromString<label::Label>(output);

		CPPUNIT_ASSERT( label == label2 );
	}
}

void LabelTest::testXMLParser() {

	label::Label label = label::labelFrom("1");

	{
	std::list<sax::Token> tokens = alib::XmlDataFactory::toTokens(label);
	std::string tmp;
	sax::SaxComposeInterface::printMemory(tmp, tokens);

	std::list<sax::Token> tokens2;
	sax::SaxParseInterface::parseMemory(tmp, tokens2);
	label::Label label2 = alib::XmlDataFactory::fromTokens<label::Label>(tokens2);

	CPPUNIT_ASSERT( label == label2 );
	}
	{
	std::string tmp = alib::XmlDataFactory::toString(label);
	label::Label label2 = alib::XmlDataFactory::fromString<label::Label>(tmp);

	CPPUNIT_ASSERT( label == label2 );
	}
}

void LabelTest::testOrder() {
	label::PrimitiveLabel s1(primitive::primitiveFrom("1"));
	label::PrimitiveLabel c1(primitive::primitiveFrom('1'));
	label::PrimitiveLabel i1(primitive::primitiveFrom(1));

	CPPUNIT_EXCLUSIVE_OR( s1 < i1, i1 < s1);
	CPPUNIT_EXCLUSIVE_OR( c1 < i1, i1 < c1);
	CPPUNIT_EXCLUSIVE_OR( s1 < c1, c1 < s1);
}

void LabelTest::testInSet() {
	{
		std::set<std::string> set1 { "a", "b", "c" };
		std::set<std::string> set2 { "a" };
		std::set<std::string> set3;

		std::set_difference(set1.begin(), set1.end(), set2.begin(), set2.end(), std::insert_iterator<std::set<std::string>>(set3, set3.begin()));
		for(const auto& item : set2) {
			set1.erase(item);
		}

		CPPUNIT_ASSERT(set1.size() == 2);
		CPPUNIT_ASSERT(set3.size() == 2);
	}
	{
		std::set<label::Label> set1 { label::labelFrom('a'), label::labelFrom('b'), label::labelFrom('c') };
		std::set<label::Label> set2 { label::labelFrom('a') };
		std::set<label::Label> set3;

		std::set_difference(set1.begin(), set1.end(), set2.begin(), set2.end(), std::insert_iterator<std::set<label::Label>>(set3, set3.begin()));
		for(const auto& item : set2) {
			set1.erase(item);
		}

		CPPUNIT_ASSERT(set1.size() == 2);
		CPPUNIT_ASSERT(set3.size() == 2);
	}
}

