#include "StdTest.h"
#include "std/variant.hpp"
#include <set>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( StdTest, "std" );
CPPUNIT_TEST_SUITE_REGISTRATION( StdTest );

void StdTest::setUp() {
}

void StdTest::tearDown() {
}

void StdTest::testIsBaseOf() {
	CPPUNIT_ASSERT( ( std::is_base_of_any<int, int, std::string, StdTest::test>::value ) == true );
	CPPUNIT_ASSERT( ( std::is_base_of<int, int>::value ) == true );
	CPPUNIT_ASSERT( ( std::is_base_of<int, std::string>::value ) == false );
	CPPUNIT_ASSERT( ( std::is_base_of<int, StdTest::test>::value ) == false );
}

void StdTest::testVariant() {
	std::variant<int, std::string, StdTest::test> d(10);
	CPPUNIT_ASSERT( d.is<int>() );
	CPPUNIT_ASSERT( d.get<int>() == 10 );
	
	std::string str = "abcde";
	d.set<std::string>(str);
	CPPUNIT_ASSERT( d.is<std::string>() );
	CPPUNIT_ASSERT( d.get<std::string>() == str );
	
	d.set<StdTest::test>(StdTest::test());
	*(d.get<StdTest::test>().holder) = 42;
	
	StdTest::test tmp;
	*(tmp.holder) = 42;
	CPPUNIT_ASSERT( d.is<StdTest::test>() );
	CPPUNIT_ASSERT( d.get<StdTest::test>() == tmp );
	
	std::variant<int, std::string, StdTest::test> e(d);
	CPPUNIT_ASSERT( e.get<StdTest::test>() == tmp );

	*(e.get<StdTest::test>().holder) = 43;

	CPPUNIT_ASSERT( (d < e) == true );
	CPPUNIT_ASSERT( (e < d) == false );
	
	CPPUNIT_ASSERT( (e == d) == false );
	
	d = std::variant<int, std::string, StdTest::test>(0);
	d = e;

	CPPUNIT_ASSERT( *(d.get<StdTest::test>().holder) == 43 );
	
	CPPUNIT_ASSERT( (d < e) == false );
	CPPUNIT_ASSERT( (e < d) == false );
	
	CPPUNIT_ASSERT( (e == d) == true );
}

void StdTest::testVariantSet() {
	StdTest::test tmp;
	*(tmp.holder) = 42;
	
	std::string str = "abcde";
	
	std::variant<int, std::string, StdTest::test> d(StdTest::test {});
	*(d.get<StdTest::test>().holder) = 42;
	std::variant<int, std::string, StdTest::test> e(d);
	
	std::variant<int, std::string, StdTest::test> f(5);
	
	std::variant<int, std::string, StdTest::test> g(str);
	
	std::set<std::variant<int, std::string, StdTest::test>> testSet;
	testSet.insert(d);
	testSet.insert(e);
	testSet.insert(f);
	testSet.insert(g);
	
	CPPUNIT_ASSERT( testSet.size() == 3 );
	
	for(std::set<std::variant<int, std::string, StdTest::test>>::const_iterator iter = testSet.begin(); iter != testSet.end(); iter++) {
		if(iter->is<int>())
			CPPUNIT_ASSERT( iter->get<int>() == 5);
		if(iter->is<std::string>())
			CPPUNIT_ASSERT( iter->get<std::string>() == str );
		if(iter->is<StdTest::test>())
			CPPUNIT_ASSERT( iter->get<StdTest::test>() == tmp );
	}
	
	std::variant<StdTest::test2, std::string, StdTest::test> h("aa");
	CPPUNIT_ASSERT( h.is<std::string>() );

	std::map<std::variant<std::string, int>, int> testMap;
	testMap.insert(std::make_pair(std::variant<std::string, int> {"aa"}, 10));

	CPPUNIT_ASSERT( testMap.size() == 1 );
	CPPUNIT_ASSERT( testMap.find(std::variant<std::string, int> {"aa"}) != testMap.end() );
	CPPUNIT_ASSERT( testMap.find(std::variant<std::string, int> {10}) == testMap.end() );

	testMap.insert(std::make_pair(std::variant<std::string, int> {"ab"}, 11));
	testMap.insert(std::make_pair(std::variant<std::string, int> {3}, 13));

	CPPUNIT_ASSERT( testMap.find(std::variant<std::string, int> {"aa"}) != testMap.end() );
	CPPUNIT_ASSERT( testMap.find(std::variant<std::string, int> {10}) == testMap.end() );
}
