#ifndef VARIANT_TEST_H_
#define VARIANT_TEST_H_

#include <cppunit/extensions/HelperMacros.h>
#include "std/compare.hpp"

class StdTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( StdTest );
  CPPUNIT_TEST( testIsBaseOf );
  CPPUNIT_TEST( testVariant );
  CPPUNIT_TEST( testVariantSet );
  CPPUNIT_TEST_SUITE_END();

public:
struct test {
	int * holder;

	test() {
		//std::cout << "test()" << std::endl;
		holder = new int();
	}

	test(test&& old) : holder(nullptr) {
		//std::cout << "test(test&&)" << std::endl;
		std::swap(holder,old.holder);
	}

	test(const test& old) {
		  //std::cout << "test(const test&)" << std::endl;
		  holder = new int(*old.holder);
	}

	~test()
	{
		//std::cout << "~test()" << std::endl;
		delete holder;
	}

	bool operator<(const test& other) const {
		return *(this->holder) < *(other.holder);
	}

	bool operator==(const test& other) const {
		return *(this->holder) == *(other.holder);
	}

	int compare(const test& other) const {
		return *(this->holder) - *(other.holder);
	}

	friend std::ostream& operator<<(std::ostream& out, const test& other) {
		out << *(other.holder);
		return out;
	}
};

struct test2 {
	int i;

	test2(int i) : i(i) {}
};

public:
  void setUp();
  void tearDown();

  void testIsBaseOf();
  void testVariant();
  void testVariantSet();
};

namespace std {

template<>
struct compare<StdTest::test> {
	int operator()(const StdTest::test& first, const StdTest::test& second) {
		return first.compare(second);
	}
};

} /* namespace std */

#endif  // VARIANT_TEST_H_
