/*
 * AllEpsilonClosure.h
 *
 *  Created on: 12. 4. 2015
 *	  Author: Jan Travnicek
 */

#ifndef EFFICIENT_ALL_EPSILON_CLOSURE_H_
#define EFFICIENT_ALL_EPSILON_CLOSURE_H_

#include <set>
#include <map>

#include "automaton/Automaton.h"
#include <automaton/common/State.h>

namespace automaton {

namespace properties {

namespace efficient {

class AllEpsilonClosure : public VisitableAutomatonBase::const_visitor_type {
public:
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::Automaton & automaton);

	/**
	 * Computes allEpsilon closure of a state in allEpsilon nonfree automaton
	 */
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::EpsilonNFA & fsm);
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::MultiInitialStateNFA & fsm);
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::NFA & fsm);
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::DFA & fsm);
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::ExtendedNFA & fsm);
	static std::map<automaton::State, std::set<automaton::State>> allEpsilonClosure( const automaton::CompactNFA & fsm);

private:
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const MultiInitialStateNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;

	static const AllEpsilonClosure ALL_EPSILON_CLOSURE;
};

} /* namespace efficient */

} /* namespace properties */

} /* namespace automaton */

#endif /* EFFICIENT_ALL_EPSILON_CLOSURE_H_ */
