/*
 * EpsilonRemoverOutgoing.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Travnicek
 */

#include "EpsilonRemoverOutgoing.h"

#include "../../properties/efficient/AllEpsilonClosure.h"

namespace automaton {

namespace simplify {

namespace efficient {

automaton::DFA EpsilonRemoverOutgoing::remove(const automaton::DFA& origFSM)
{
	return origFSM;
}

automaton::MultiInitialStateNFA EpsilonRemoverOutgoing::remove(const automaton::MultiInitialStateNFA& origFSM)
{
	return origFSM;
}

automaton::NFA EpsilonRemoverOutgoing::remove(const automaton::NFA& origFSM)
{
	return origFSM;
}

automaton::MultiInitialStateNFA EpsilonRemoverOutgoing::remove( const automaton::EpsilonNFA & origFSM ) {
	automaton::MultiInitialStateNFA fsm;
	fsm.setStates( origFSM.getStates() );
	fsm.setFinalStates( origFSM.getFinalStates() );
	fsm.setInputAlphabet( origFSM.getInputAlphabet() );

	std::map<automaton::State, std::set<automaton::State>> closures = automaton::properties::efficient::AllEpsilonClosure::allEpsilonClosure( origFSM );

	std::map<std::pair<automaton::State, alphabet::Symbol>, std::set<automaton::State>> transitions = origFSM.getSymbolTransitions();

	for(const std::pair<std::pair<automaton::State, alphabet::Symbol>, std::set<automaton::State>>& transition : transitions)
		for(const automaton::State& to : transition.second)
			for(const automaton::State& toClosure : closures[to])
				fsm.addTransition(transition.first.first, transition.first.second, toClosure);

	/**
	 * Step 2 from Melichar 2.41
	 */
	const std::set<automaton::State>& cl = closures [ origFSM.getInitialState() ];
	fsm.setInitialStates( cl );

	return fsm;
}

automaton::Automaton EpsilonRemoverOutgoing::remove(const automaton::Automaton& automaton) {
	automaton::Automaton* out = NULL;
	automaton.getData().Accept((void*) &out, EpsilonRemoverOutgoing::EPSILON_REMOVER_OUTGOING);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

void EpsilonRemoverOutgoing::Visit(void* data, const automaton::EpsilonNFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->remove(automaton));
}

void EpsilonRemoverOutgoing::Visit(void* data, const automaton::MultiInitialStateNFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->remove(automaton));
}

void EpsilonRemoverOutgoing::Visit(void* data, const automaton::NFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->remove(automaton));
}

void EpsilonRemoverOutgoing::Visit(void* data, const automaton::DFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->remove(automaton));
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::ExtendedNFA& ) const {
	throw exception::AlibException("Unsupported automaton type ExtendedNFA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::CompactNFA& ) const {
	throw exception::AlibException("Unsupported automaton type CompactNFA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::DPDA&) const {
	throw exception::AlibException("Unsupported automaton type DPDA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::SinglePopDPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::InputDrivenDPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenDPDA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::InputDrivenNPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::VisiblyPushdownDPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::VisiblyPushdownNPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::RealTimeHeightDeterministicDPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::RealTimeHeightDeterministicNPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::NPDA&) const {
	throw exception::AlibException("Unsupported automaton type NPDA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::SinglePopNPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void EpsilonRemoverOutgoing::Visit(void*, const automaton::OneTapeDTM&) const {
	throw exception::AlibException("Unsupported automaton type OneTapeDTM");
}

const EpsilonRemoverOutgoing EpsilonRemoverOutgoing::EPSILON_REMOVER_OUTGOING;

} /*namespace efficient */

} /* namespace simplify */

} /* namespace automaton */
