/*
 * EpsilonRemoverOutgoing.h
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Travnicek
 */

#ifndef EFFICIENT_EPSILON_REMOVER_OUTGOING_H_
#define EFFICIENT_EPSILON_REMOVER_OUTGOING_H_

#include <map>
#include <algorithm>

#include <automaton/Automaton.h>

#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>
#include <exception/AlibException.h>

namespace automaton {

namespace simplify {

namespace efficient {

class EpsilonRemoverOutgoing : public automaton::VisitableAutomatonBase::const_visitor_type {
public:
	static automaton::Automaton remove( const automaton::Automaton & automaton );

	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	static automaton::MultiInitialStateNFA remove( const automaton::EpsilonNFA & fsm );
	static automaton::MultiInitialStateNFA remove( const automaton::MultiInitialStateNFA & fsm );
	static automaton::NFA remove( const automaton::NFA & fsm );
	static automaton::DFA remove( const automaton::DFA & fsm );

private:
	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::MultiInitialStateNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;
	void Visit(void*, const automaton::DPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenNPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const automaton::NPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void*, const automaton::OneTapeDTM& automaton) const;

	static const EpsilonRemoverOutgoing EPSILON_REMOVER_OUTGOING;
};

} /* namespace efficient */

} /* namespace simplify */

} /* namespace automaton */

#endif /* EFFICIENT_EPSILON_REMOVER_OUTGOING_H_ */
