/*
 * aminimize.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>

#include <exception/AlibException.h>
#include <factory/XmlDataFactory.hpp>

#include "automaton/simplify/Minimize.h"
#include "automaton/simplify/MinimizeBrzozowski.h"

int main(int argc, char** argv) {

	try {
		TCLAP::CmdLine cmd("Automaton minimize binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Automaton to minimize",	false,	"-",		"file");
		cmd.add( input );

		std::vector<std::string> algorithms {"hopcroft", "brzozowski" };
		TCLAP::ValuesConstraint<std::string> algorithmVals( algorithms );
		TCLAP::ValueArg<std::string> algorithm(	"a",	"algorithm",	"Use specific algorithm. Hopcroft default.",	false,	"hopcroft",	&algorithmVals);
		cmd.add( algorithm );

		cmd.parse(argc, argv);

		std::list<sax::Token> tokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		if(algorithm.getValue() == "hopcroft") {
			alib::XmlDataFactory::toStdout(automaton::simplify::Minimize::minimize(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
		} else if(algorithm.getValue() == "brzozowski") {
			alib::XmlDataFactory::toStdout(automaton::simplify::MinimizeBrzozowski::minimize(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
		} else {
			throw exception::AlibException("Unknown algorithm selected.");
		}

		return 0;

	} catch (const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch (const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch (...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
