/*
 * anormalize.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>

#include "exception/AlibException.h"
#include "factory/XmlDataFactory.hpp"
#include "string/simplify/NormalizeAlphabet.h"
#include "automaton/simplify/Normalize.h"
#include "automaton/simplify/SingleInitialState.h"
#include "grammar/convert/ToGrammarLeftRG.h"
#include "grammar/convert/ToGrammarRightRG.h"
#include "grammar/simplify/ToCNF.h"
#include "grammar/simplify/ToGNF.h"

int main(int argc, char** argv) {

	try {
		TCLAP::CmdLine cmd("Automaton normalize binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input to normalize",		false,	"-",	"file");
		cmd.add( input );

		std::vector<std::string> forms {"leftRG", "rightRG", "singleInitialState", "CNF", "GNF" };
		TCLAP::ValuesConstraint<std::string> formVals( forms );

		TCLAP::ValueArg<std::string> form(	"f",	"form",		"Convert to different form",	false,	"",	&formVals);
		cmd.add( form );

		std::vector<std::string> inputTypes {"automaton", "grammar"};
		TCLAP::ValuesConstraint<std::string> inputTypeVals( inputTypes );

		TCLAP::ValueArg<std::string> labels(	"l",	"labels",	"Normalize labels",		false,	"",	&inputTypeVals);
		cmd.add( labels );

		std::vector<std::string> inputTypes2 {"string"};
		TCLAP::ValuesConstraint<std::string> inputTypeVals2( inputTypes2 );

		TCLAP::ValueArg<std::string> alphabet(	"a",	"alphabet",	"Normalize alphabet",		false,	"",	&inputTypeVals2);
		cmd.add( alphabet );

		cmd.parse(argc, argv);

		std::list<sax::Token> tokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		if(labels.getValue() == "automaton") {
			alib::XmlDataFactory::toStdout(automaton::simplify::Normalize::normalize(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
			return 0;
		} else if(labels.getValue() == "grammar") {
//			alib::XmlDataFactory::toStdout(automaton::simplify::Normalize::normalize(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
			return 0;
		} else if(alphabet.getValue() == "string") {
			alib::XmlDataFactory::toStdout(string::simplify::NormalizeAlphabet::normalize(alib::XmlDataFactory::fromTokens<string::String>(tokens)));
			return 0;
		} else if(form.getValue() == "leftRG") {
			alib::XmlDataFactory::toStdout(grammar::convert::ToGrammarLeftRG::convert(alib::XmlDataFactory::fromTokens<grammar::Grammar>( tokens )));
			return 0;
		} else if(form.getValue() == "rightRG") {
			alib::XmlDataFactory::toStdout(grammar::convert::ToGrammarRightRG::convert(alib::XmlDataFactory::fromTokens<grammar::Grammar>( tokens )));
			return 0;
		} else if(form.getValue() == "singleInitialState") {
			alib::XmlDataFactory::toStdout(automaton::simplify::SingleInitialState::convert(alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens)));
			return 0;
		} else if(form.getValue() == "CNF") {
			alib::XmlDataFactory::toStdout(grammar::simplify::ToCNF::convert(alib::XmlDataFactory::fromTokens<grammar::Grammar>( tokens )));
			return 0;
		} else if(form.getValue() == "GNF") {
			alib::XmlDataFactory::toStdout(grammar::simplify::ToGNF::convert(alib::XmlDataFactory::fromTokens<grammar::Grammar>( tokens )));
			return 0;
		} else {
			throw exception::AlibException("Invalid normalize command");
		}
	} catch (const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}

