/*
 * arun.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <vector>

#include <factory/XmlDataFactory.hpp>
#include <exception/AlibException.h>
#include <string/String.h>
#include <automaton/Automaton.h>
#include <automaton/run/Accept.h>
#include <automaton/run/Result.h>
#include <automaton/run/Occurrences.h>

int main(int argc, char* argv[]) {
	try {
		TCLAP::CmdLine cmd("Automaton run binary", ' ', "0.01");

		std::vector<std::string> allowed;
		allowed.push_back("occurrences");
		allowed.push_back("accept");
		allowed.push_back("result");
		TCLAP::ValuesConstraint<std::string> allowedVals( allowed );

		TCLAP::ValueArg<std::string> type(	"t",	"type",		"Run type",		false,	"accept",	&allowedVals);
		cmd.add( type );

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input from file",	false,	"-",		"file");
		cmd.add( input );

		TCLAP::ValueArg<std::string> automaton(	"a",	"automaton",	"Automaton to run",	false,	"-",		"file");
		cmd.add( automaton );

		cmd.parse(argc, argv);

		std::list<sax::Token> inputTokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(inputTokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), inputTokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(inputTokens);
		}

		std::list<sax::Token> automatonTokens;
		if(automaton.isSet()) {
			if(automaton.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(automatonTokens);
			} else {
				sax::SaxParseInterface::parseFile(automaton.getValue(), automatonTokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(automatonTokens);
		}

		if( type.getValue() == "occurrences") {
			string::LinearString input = alib::XmlDataFactory::fromTokens<string::LinearString>(inputTokens);
			automaton::Automaton automaton = alib::XmlDataFactory::fromTokens<automaton::Automaton>(automatonTokens);
			std::set<unsigned> res = automaton::run::Occurrences::occurrences(automaton, input);
			alib::XmlDataFactory::toStdout( res );
			return 0;
		} else if( type.getValue() == "accept") {
			string::LinearString input = alib::XmlDataFactory::fromTokens<string::LinearString>(inputTokens);
			automaton::Automaton automaton = alib::XmlDataFactory::fromTokens<automaton::Automaton>(automatonTokens);
			bool res = automaton::run::Accept::accept(automaton, input);
			alib::XmlDataFactory::toStdout( res );
		} else if( type.getValue() == "result") {
			string::LinearString input = alib::XmlDataFactory::fromTokens<string::LinearString>(inputTokens);
			automaton::Automaton automaton = alib::XmlDataFactory::fromTokens<automaton::Automaton>(automatonTokens);
			label::Label res = automaton::run::Result::result(automaton, input);
			alib::XmlDataFactory::toStdout( res );
		} else {
			throw exception::AlibException( "Invalid run type" );
			return 1;
		}
	} catch ( const exception::AlibException & exception ) {
		alib::XmlDataFactory::toStdout( exception );
		return 1;
	} catch (const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch (...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
